/*
 *  Convert.c
 *  Version 2.0 / 000911
 *  by Satoshi Takahashi
 */

#ifndef CONVERT_C /*pragma once*/
#define CONVERT_C
#ifndef SCLLocal
#include <s_Config.h>
#else
#include "s_Config.h"
#endif
#if S_CARBON
#include <Carbon/Carbon.h>
#endif

#include <ctype.h>
#include <string.h>

#ifndef SCLLocal
#include <s_String.h>

#include <Convert.h>
#else
#include "s_String.h"

#include "Convert.h"
#endif

enum {JISASCII = 1, JISKanji};
#ifndef ESC
#define ESC '\x1b'
#endif


void RemoveTopLF(char *oneLine)
{
int i;

	if (oneLine[0] == '\r')
	{
		for (i = 0; oneLine[i] = oneLine[i+1]; i++)
			;
	}
}


int DecodeHeaderMime(char *inBuf)
{
char *outBuf;
unsigned char c0, c1, c2, c3;
int count0, count1, count2, count3;
int length;
#if DEBUG_SCRIPT
char *outBufHead;
#endif
	outBuf = inBuf;
	
#if DEBUG_SCRIPT
outBufHead = outBuf;
#endif
	while(1)
	{
		while (!HeadStrCL("=?iso-2022-jp?B?", inBuf))
		{
			if (!(*outBuf++ = *inBuf++))
				return 0;
		}
#if DEBUG_SCRIPT
s_printf("inBuf: %s", inBuf);Stop();
s_printf("outBufHead: %s", outBufHead);Stop();
#endif		
	//printf("Begin Heder Mime Decode\n");Stop();	
		inBuf += strlen("=?iso-2022-jp?B?");
		
		length = strlen(inBuf);

		for (; length > 4 && inBuf[0] != '?'; inBuf += 4)
		{
			c0 = MimeChar(inBuf[0], &count0);
			c1 = MimeChar(inBuf[1], &count1);
			c2 = MimeChar(inBuf[2], &count2);
			c3 = MimeChar(inBuf[3], &count3);

			outBuf[0] = (c0 << 2) + (c1 >> 4);
			if (count2)
				outBuf[1] = (c1 << 4) + (c2 >> 2);
			if (count3)
				outBuf[2] = (c2 << 6) + (c3 >> 0);
			outBuf += 1 + count2 + count3;
		}
		
		if (inBuf[0] != '?')
			return -1; /*error*/

		inBuf += 2;	/*"?="*/
#if DEBUG_SCRIPT
s_printf("inBuf: %s", inBuf);Stop();
s_printf("outBufHead: %s", outBufHead);Stop();
#endif		
		//while (*outBuf++ = *inBuf++)
			//;
	}
	return 0; /*do not reach here*/
}


unsigned char MimeChar(char c, int *countP)
{
	*countP = 1;
	
	if (isupper(c))
		return c - 'A';
	else if (islower(c))
		return c - 'a' + ('Z' - 'A' + 1);
	else if (isdigit(c))
		return c - '0' + ('Z' - 'A' + 1) + ('z' - 'a' + 1);
	else if (c == '+')
		return 62;
	else if (c == '/')
		return 63;
	else if (c == '=')
	{
		*countP = 0;
		return 0;
	}
	
	*countP = -1; /* error */
	return 0xff;
}


int ConvertJIS2SJIS(char *inBuf)
{
char *outBuf;
int state = JISASCII;

	outBuf = inBuf;
	
	while(inBuf[0])
	{
		if (inBuf[0] == ESC)
		{
			state = JISState(inBuf[1], inBuf[2]);
			if (state < 0)
				return state; /*error*/
			
			inBuf += 3;
		}
		
		if (state == JISKanji)
		{
			while (inBuf[0] && inBuf[1] && (inBuf[0] != ESC))
			{
				KuTen2SJIS(inBuf[0] - 0x20, inBuf[1] - 0x20,
								&outBuf[0], &outBuf[1]);
				inBuf += 2;
				outBuf += 2;
			}
			if (inBuf[0] != ESC)
				return -1; /*error*/
		}
		else
		{
			while (inBuf[0] && (inBuf[0] != ESC))
				*outBuf++ = *inBuf++;
		}
	}
	outBuf[0] = 0; /*terminate*/
	
	return 0; /*no error*/
}


int JISState(char c0, char c1)
{
	if ( (c0 == '$') && ( (c1 == 'B') /* New JIS */
		  			 || (c1 == '@') /* Old JIS */) )
	{
		return JISKanji;
	}
	else if ( (c0 == '(')
		&& ( (c1 == 'B') /* ASCII */
		  || (c1 == 'J') /* Romaji */) )
	{
		return JISASCII;
	}
	else
		return -1; /*error*/
}


void KuTen2SJIS(unsigned char ku, unsigned char ten, char *c0, char *c1)
{
	if ((ku < 1) || (94 < ku) || (ten < 1) || (94 < ten))
	{
		*c0 = (2 - 1)/2 + 129; /* Geta */
		*c1 = 14 + 158;
		return;
	}

	if ( ku <= 62 )
		*c0 = (ku - 1)/2 + 129;
	else
		*c0 = (ku - 1)/2 + 193;
		
	if ( ku % 2 )
		*c1 = ten + 63 + (ten >= 64);
	else
		*c1 = ten + 158;
}

#endif /*pragma once*/