/*
 *  Formula-Calc.c
 *  Version 1.5.1 / 980129
 *  by Satoshi Takahashi and Jiro Kikuchi
 */


#ifndef FORMULA_CALC_C  /*pragma once*/
#define FORMULA_CALC_C


/************************ Switches *******************/

#include <Formula-Switches.h>

/************************ Include Headers *******************/

#include <ctype.h>
#include <stdlib.h>

#include <s_Common.h>
#include <StartSCL.h>
#include <s_String.h>
#include <s_Debug.h>

#include <Formula-Common.h>
#include <Formula.h>
#include <Formula-v.h>
#include <Formula-Calc.h>

#ifndef S_PROFILE
#if PROFILE_FORMULA
#define S_PROFILE 1
#else
#define S_PROFILE 0
#endif
#endif

#include <s_Profile.h>


/*************************** Expand and Sort **************************/

void SortFracFormula(void)
{
int fNum = 0; /* current formula number */

	SkipSpaces(&s_f_str);
	fNum = FormulaNumber(s_f_str);
	if (fNum >= s_f_fNum)
	{
		s_printf("Formula NO. %d is out of range.\n",fNum);
		return;
	}
	s_f_curFNum = fNum;

StartProfile(0);

	SortFracNumFormula(fNum);

StopProfile(0);
PrintProfile(0);
ResetProfile();
	
	s_f_formulas[fNum].sorted = 1;
	PrintNumFormula(fNum);
}

void SortFracNumFormula(int fNum)
{
s_f_FB *lastFB, *denomFB;

	if (!(s_f_formulas[fNum].expanded))
	{
		ExpandFB(&s_f_formulas[fNum].fb, &lastFB, &denomFB);
		if (denomFB)
		{
			s_f_formulas[fNum].fb = ComposeFrac(s_f_formulas[fNum].fb, denomFB);
			s_f_formulas[fNum].divDepth = s_f_formulas[fNum].frac = 1;
		}
		s_f_formulas[fNum].expanded = 1;
	}
	if (s_f_formulas[fNum].frac)
	{
		SortFormulaBlock(&(s_f_formulas[fNum].fb->block));
		SortFormulaBlock(&(s_f_formulas[fNum].fb->next->block));
	}
	else
	{
		SortFormulaBlock(&(s_f_formulas[fNum].fb));
	}
}

void ExpandFB(s_f_FB** orgFBP, s_f_FB** retLastFBP,s_f_FB** retDenomFBP)
{
s_f_FB *curFB, *newTopFB, *newLastFB, *newDenomFB = 0;
int power;



	dwf("ExpandFB 0\n");
	curFB = *orgFBP;
	power = orgFBP[0]->power;

	/* set top blocks of new block */	
	switch (curFB->type)
	{
		case BaseType:
				ExpandTopBase(curFB, &newTopFB, &newLastFB);
			break;
		case BlockType:
				ExpandTopBlock(curFB, &newTopFB,
								&newLastFB, &newDenomFB);
			break;
	}
	NextFB(&curFB);
	
	/* expand polynomial block */	
	while(curFB && (!newDenomFB))
	{
		switch (curFB->type)
		{
			case BaseType:
				ExpandNoFracOneBase(curFB,
							&newTopFB, &newLastFB, &newDenomFB);
				break;
			case BlockType:
					ExpandNoFracOneBlock(curFB,
							&newTopFB, &newLastFB, &newDenomFB);
				break;
		}
		NextFB(&curFB);
	}
	/* expand fractional block */	
	while(curFB)
	{
		switch (curFB->type)
		{
			case BaseType:
					ExpandFracOneBase(curFB,
							&newTopFB, &newLastFB, &newDenomFB);
				break;
			case BlockType:
					ExpandFracOneBlock(curFB,
							&newTopFB, &newLastFB, &newDenomFB);
				break;
		}
		NextFB(&curFB);
	}
	
	/* power block */
	if (power > 1)
	{
		ExpandPowerBlock(&newTopFB,&newLastFB,&newDenomFB, power);
	}
	
	/* return block */
	*orgFBP = newTopFB;
	*retLastFBP = newLastFB;
	*retDenomFBP = newDenomFB;
	dwf("ExpandFB END\n");
}

void ExpandTopBase(s_f_FB* curFB, s_f_FB** newTopFBP, s_f_FB** newLastFBP)
{
	CopyOneBlock(curFB,newTopFBP);
	*newLastFBP = *newTopFBP;
}

void ExpandTopBlock(s_f_FB* curFB, s_f_FB** newTopFBP,
					s_f_FB** newLastFBP, s_f_FB** newDenomFBP)
{
	newTopFBP[0] = curFB->block;
	ExpandFB(newTopFBP, newLastFBP, newDenomFBP);
	if (curFB->operand == SubOp)
		InvertSigns(newTopFBP);
}

void ExpandNoFracOneBase(s_f_FB* curFB, s_f_FB** newTopFBP,
					s_f_FB** newLastFBP, s_f_FB** newDenomFBP)
{

s_f_FB *oldTopFB, *oldLastFB;


	switch (curFB->operand)
	{
	case AddOp:
	case SubOp:
			AppendOneBlock(curFB,newLastFBP);
		break;
	case MulOp:
		if (s_f_GCFlag)
		{
			oldTopFB = *newTopFBP;
			oldLastFB = *newLastFBP;
			*newTopFBP = MultiplyPlusBlockAndBase(newTopFBP,
									&(curFB->base), newLastFBP);
#if !DEBUG_FORMULA
			FreeBaseFBsWithLast(oldTopFB,oldLastFB);
#endif
		}
		else
			*newTopFBP = MultiplyPlusBlockAndBase(newTopFBP,
							&(curFB->base), newLastFBP);
		break;
	case DivOp:
		CopyOneBlock(curFB,newDenomFBP);
		newDenomFBP[0]->operand = AddOp;
		break;
	}	
}

void ExpandNoFracOneBlock(s_f_FB* curFB, s_f_FB** newTopFBP,
					s_f_FB** newLastFBP, s_f_FB** newDenomFBP)
{
s_f_FB *curBlockFB, *curBlockLastFB, *curBlockDenomFB = 0;

s_f_FB *oldTopFB, *oldLastFB;

	curBlockFB = curFB->block;

	ExpandFB(&curBlockFB, &curBlockLastFB, &curBlockDenomFB);

	
	if (!curBlockDenomFB) /* Polynomial */
	{
		switch (curFB->operand)
		{
		case SubOp:
			InvertSigns(&curBlockFB);
		case AddOp:
			AddBlocksWithLast(newLastFBP, curBlockFB, curBlockLastFB);
			break;
		case MulOp:
			if (s_f_GCFlag)
			{
				oldTopFB = *newTopFBP;
				oldLastFB = *newLastFBP;
				*newTopFBP = MultiplyPlusBlocks(newTopFBP,&curBlockFB,newLastFBP);
				FreeBaseFBsWithLast(oldTopFB,oldLastFB);
				FreeBaseFBsWithLast(curBlockFB,curBlockLastFB);
			}
			else
				*newTopFBP = MultiplyPlusBlocks(newTopFBP,&curBlockFB,newLastFBP);

			break;
		case DivOp:
			*newDenomFBP = curBlockFB;
			break;
		}	
	}
	else /* fractional */
	{
		switch (curFB->operand)
		{
		case SubOp:
				InvertSigns(&curBlockFB);
		case AddOp:
			if (s_f_GCFlag)
			{
				oldTopFB = *newTopFBP;
				oldLastFB = *newLastFBP;
				*newTopFBP = MultiplyPlusBlocks(newTopFBP,&curBlockDenomFB,newLastFBP);
				FreeBaseFBsWithLast(oldTopFB,oldLastFB);
			}
			else
				*newTopFBP = MultiplyPlusBlocks(newTopFBP,&curBlockDenomFB,newLastFBP);

			AddBlocksWithLast(newLastFBP, curBlockFB, curBlockLastFB);
			*newDenomFBP = curBlockDenomFB;
			break;
		case MulOp:
			if (s_f_GCFlag)
			{
				oldTopFB = *newTopFBP;
				oldLastFB = *newLastFBP;
				*newTopFBP = MultiplyPlusBlocks(newTopFBP,&curBlockFB,newLastFBP);
				FreeBaseFBsWithLast(oldTopFB,oldLastFB);
				FreeBaseFBsWithLast(curBlockFB,curBlockLastFB);
			}
			else
				*newTopFBP = MultiplyPlusBlocks(newTopFBP,&curBlockFB,newLastFBP);

			*newDenomFBP = curBlockDenomFB;
			break;
		case DivOp:
			if (s_f_GCFlag)
			{
				oldTopFB = *newTopFBP;
				oldLastFB = *newLastFBP;
				*newTopFBP = MultiplyPlusBlocks(newTopFBP,&curBlockDenomFB,newLastFBP);
				FreeBaseFBsWithLast(oldTopFB,oldLastFB);
				FreeBaseFBs(curBlockDenomFB);
			}
			else
				*newTopFBP = MultiplyPlusBlocks(newTopFBP,&curBlockDenomFB,newLastFBP);

			*newDenomFBP = curBlockFB;
			break;
		}	
	}
}

void ExpandFracOneBase(s_f_FB* curFB, s_f_FB** newTopFBP,
					s_f_FB** newLastFBP, s_f_FB** newDenomFBP)
{
s_f_FB *curNumFB, *curLastFB;
s_f_FB *oldTopFB, *oldLastFB, *oldDenomFB;

	switch (curFB->operand)
	{
	case SubOp:
	case AddOp:
		curNumFB = MultiplySignedBlockAndBase(newDenomFBP,
						&(curFB->base), &curLastFB, curFB->operand);
		AddBlocksWithLast(newLastFBP, curNumFB, curLastFB);
		break;
	case MulOp:
		if (s_f_GCFlag)
		{
			oldTopFB = *newTopFBP;
			oldLastFB = *newLastFBP;
			*newTopFBP = MultiplyPlusBlockAndBase(newTopFBP,
							&(curFB->base), newLastFBP);
			FreeBaseFBsWithLast(oldTopFB,oldLastFB);
		}
		else
			*newTopFBP = MultiplyPlusBlockAndBase(newTopFBP,
							&(curFB->base), newLastFBP);
		break;
	case DivOp:
		if (s_f_GCFlag)
		{
			oldDenomFB = *newDenomFBP;
			*newDenomFBP = MultiplyPlusBlockAndBase(newDenomFBP,
							&(curFB->base), newLastFBP);
			FreeBaseFBs(oldDenomFB);
		}
		else
			*newDenomFBP = MultiplyPlusBlockAndBase(newDenomFBP,
							&(curFB->base), newLastFBP);
		break;
	}	
}

void ExpandFracOneBlock(s_f_FB* curFB, s_f_FB** newTopFBP,
					s_f_FB** newLastFBP, s_f_FB** newDenomFBP)
{
s_f_FB *curBlockFB, *curBlockLastFB, *curBlockDenomFB = 0, *dummyFB = 0;
s_f_FB *oldTopFB, *oldLastFB, *oldDenomFB, *oldCurFB, *oldCurLastFB, *oldCurDenomFB;

	curBlockFB = curFB->block;
	ExpandFB(&curBlockFB, &curBlockLastFB, &curBlockDenomFB);
	
	if (!curBlockDenomFB) /* Polynomial */
	{
		switch (curFB->operand)
		{
		case AddOp:
		case SubOp:
			if (s_f_GCFlag)
			{
				oldCurFB = curBlockFB;
				oldCurLastFB = curBlockLastFB;
				curBlockFB = MultiplySignedBlocks(newDenomFBP, &curBlockFB,
	 				&curBlockLastFB, curFB->operand);
				FreeBaseFBsWithLast(oldCurFB,oldCurLastFB);
			}
			else
				curBlockFB = MultiplySignedBlocks(newDenomFBP, &curBlockFB,
 					&curBlockLastFB, curFB->operand);
			AddBlocksWithLast(newLastFBP, curBlockFB, curBlockLastFB);
			break;
		case MulOp:
			if (s_f_GCFlag)
			{
				oldTopFB = *newTopFBP;
				oldLastFB = *newLastFBP;
				*newTopFBP = MultiplyPlusBlocks(newTopFBP,&curBlockFB,newLastFBP);
				FreeBaseFBsWithLast(oldTopFB,oldLastFB);
				FreeBaseFBsWithLast(curBlockFB,curBlockLastFB);
			}
			else
				*newTopFBP = MultiplyPlusBlocks(newTopFBP,&curBlockFB,newLastFBP);
			break;
		case DivOp:
			if (s_f_GCFlag)
			{
				oldDenomFB = *newDenomFBP;
				*newDenomFBP = MultiplyPlusBlocks(newDenomFBP,&curBlockFB,newLastFBP);
				FreeBaseFBs(oldDenomFB);
				FreeBaseFBsWithLast(curBlockFB,curBlockLastFB);
			}
			else
				*newDenomFBP = MultiplyPlusBlocks(newDenomFBP,&curBlockFB,newLastFBP);
			break;
		}	
	}
	else /* fractional */
	{
		switch (curFB->operand)
		{
		case AddOp:
		case SubOp:
			if (s_f_GCFlag)
			{
				oldTopFB = *newTopFBP;
				oldLastFB = *newLastFBP;
				oldDenomFB = *newDenomFBP;
				
				oldCurFB = curBlockFB;
				oldCurLastFB = curBlockLastFB;
				oldCurDenomFB = curBlockDenomFB;

				*newTopFBP = MultiplyPlusBlocks(newTopFBP,&curBlockDenomFB,newLastFBP);
				curBlockFB = MultiplySignedBlocks(&curBlockFB,newDenomFBP,
											&curBlockLastFB, curFB->operand);
				AddBlocksWithLast(newLastFBP, curBlockFB, curBlockLastFB);
				*newDenomFBP = MultiplyPlusBlocks(newDenomFBP,&curBlockDenomFB,&dummyFB);

				FreeBaseFBsWithLast(oldTopFB,oldLastFB);
				FreeBaseFBsWithLast(oldCurFB,oldCurLastFB);
				
				FreeBaseFBs(oldDenomFB);
				FreeBaseFBs(oldCurDenomFB);
			}
			else
			{
				*newTopFBP = MultiplyPlusBlocks(newTopFBP,&curBlockDenomFB,newLastFBP);
				curBlockFB = MultiplySignedBlocks(&curBlockFB,newDenomFBP,
											&curBlockLastFB, curFB->operand);
				AddBlocksWithLast(newLastFBP, curBlockFB, curBlockLastFB);
				*newDenomFBP = MultiplyPlusBlocks(newDenomFBP,&curBlockDenomFB,&dummyFB);
			}
			break;
		case MulOp:
			if (s_f_GCFlag)
			{
				oldTopFB = *newTopFBP;
				oldLastFB = *newLastFBP;
				oldDenomFB = *newDenomFBP;
				
				oldCurFB = curBlockFB;
				oldCurLastFB = curBlockLastFB;
				oldCurDenomFB = curBlockDenomFB;

				*newTopFBP = MultiplyPlusBlocks(newTopFBP,&curBlockFB,newLastFBP);
				*newDenomFBP = MultiplyPlusBlocks(newDenomFBP,&curBlockDenomFB,&dummyFB);

				FreeBaseFBsWithLast(oldTopFB,oldLastFB);
				FreeBaseFBsWithLast(oldCurFB,oldCurLastFB);
				
				FreeBaseFBs(oldDenomFB);
				FreeBaseFBs(oldCurDenomFB);
			}
			else
			{
				*newTopFBP = MultiplyPlusBlocks(newTopFBP,&curBlockFB,newLastFBP);
				*newDenomFBP = MultiplyPlusBlocks(newDenomFBP,&curBlockDenomFB,&dummyFB);
			}
			break;
		case DivOp:
			if (s_f_GCFlag)
			{
				oldTopFB = *newTopFBP;
				oldLastFB = *newLastFBP;
				oldDenomFB = *newDenomFBP;
				
				oldCurFB = curBlockFB;
				oldCurLastFB = curBlockLastFB;
				oldCurDenomFB = curBlockDenomFB;

				*newTopFBP = MultiplyPlusBlocks(newTopFBP,&curBlockDenomFB,newLastFBP);
				*newDenomFBP = MultiplyPlusBlocks(newDenomFBP,&curBlockFB,&dummyFB);

				FreeBaseFBsWithLast(oldTopFB,oldLastFB);
				FreeBaseFBsWithLast(oldCurFB,oldCurLastFB);
				
				FreeBaseFBs(oldDenomFB);
				FreeBaseFBs(oldCurDenomFB);
			}
			else
			{
				*newTopFBP = MultiplyPlusBlocks(newTopFBP,&curBlockDenomFB,newLastFBP);
				*newDenomFBP = MultiplyPlusBlocks(newDenomFBP,&curBlockFB,&dummyFB);
			}
			break;
		}	
	}
}

void ExpandPowerBlock(s_f_FB** newTopFBP, s_f_FB** newLastFBP,
								s_f_FB** newDenomFBP, int power)
{
s_f_FB *mulNumFB, *mulDenomFB, *dummyFB;
s_f_FB *newMulNumFB, *newMulDenomFB;

	mulNumFB = MultiplyBlocks(newTopFBP,newTopFBP,newLastFBP);
	power--;
	
	if (!(*newDenomFBP)) /* Polynomial */
	{
		while(power > 1)
		{
			if (s_f_GCFlag)
			{
				newMulNumFB = MultiplyBlocks(&mulNumFB,newTopFBP,newLastFBP);
				FreeBaseFBs(mulNumFB);
				mulNumFB = newMulNumFB;
			}
			else
				mulNumFB = MultiplyBlocks(&mulNumFB,newTopFBP,newLastFBP);
			power--;
		}
	}
	else /* Fractional */
	{
		mulDenomFB = MultiplyBlocks(newDenomFBP,newDenomFBP,&dummyFB);
		while(power > 1)
		{
			if (s_f_GCFlag)
			{
				newMulNumFB = MultiplyBlocks(&mulNumFB,newTopFBP,newLastFBP);
				FreeBaseFBs(mulNumFB);
				mulNumFB = newMulNumFB;

				newMulDenomFB = MultiplyBlocks(&mulDenomFB,newDenomFBP,&dummyFB);
				FreeBaseFBs(mulDenomFB);
				mulDenomFB = newMulDenomFB;
			}
			else
			{
				mulNumFB = MultiplyBlocks(&mulNumFB,newTopFBP,newLastFBP);
				mulDenomFB = MultiplyBlocks(&mulDenomFB,newDenomFBP,&dummyFB);
			}				
			power--;
		}
		newDenomFBP[0] = mulDenomFB;
	}
	
	newTopFBP[0] = mulNumFB;
}



void SortFormulaBlock(s_f_FB **orgFBP) /* change variable names */
{
s_f_FB *inFB, *outFB, *curInFB = 0, *curOutFB = 0,
			*prevCurOutFB = 0;
int cmpResult = 0;

	dwf("SortFormulaBlock 0\n");
	inFB = *orgFBP;
	curInFB = inFB->next;
	outFB = inFB;
	outFB->next = 0;

	while(curInFB)
	{

		prevCurOutFB = 0;
		curOutFB = outFB;

		while(curOutFB) /* find where to insert curInFB */
		{
			cmpResult = (*s_f_compareBasesP)(curInFB->base->power, curOutFB->base->power);
		loopcont2: ;
			switch (cmpResult)
			{
			case 1: /* curInFB->base > curOutFB->base */
				prevCurOutFB = curOutFB;
				curOutFB = curOutFB->next;
				break;
			case -1: /* curInFB->base < curOutFB->base */
				goto loopbottom1;
				break;
			case 0: /* curInFB->base = curOutFB->base */
				if(curOutFB->operand == curInFB->operand)
				{
					curOutFB->base->coef += curInFB->base->coef;
				}
				else
				{
					curOutFB->base->coef -= curInFB->base->coef;
				}
				if (s_f_GCFlag)
				{
					FreeOneBaseFB(&curInFB);
				}
				else
					curInFB = curInFB->next;
				goto loopbottom2;
				break;	
			}
		}
loopbottom1:
		InsertFB(prevCurOutFB, &curInFB, curOutFB, &outFB);
loopbottom2: ;

CheckOrderFBs(outFB);
		if (curOutFB)
		{
			prevCurOutFB = curOutFB->prev;
		}
		else
		{
			curOutFB = prevCurOutFB->next;
		}
CheckOrder(prevCurOutFB);
		if (!curInFB)
			break;

			cmpResult = (*s_f_compareBasesP)(curInFB->base->power, curOutFB->base->power);
		if (cmpResult != -1)
		{
			goto loopcont2;
		}
	
	}
	FBSignOp(&outFB);
	*orgFBP = outFB;
	dwf("SortFormulaBlock END\n");
}



/**************************** Truncate ***********************************/

void TruncateFormula()
{
int inputNum;
char formulaString[S_F_MAX_NAME_LENGTH+1];
char symbolString[S_F_MAX_NAME_LENGTH+1];
int power;
int formulaNum;
int symbolNum;
s_f_FB *numFB, *denomFB;

	SkipSpacesAndReadCont();
	inputNum = sscanf(s_f_str, "%s %s %d", formulaString, symbolString, &power);

	/* check input format */
	
	if (TruncateCheckInput(inputNum, formulaString, symbolString, power)) /* error in input */
	{
		s_printf("Error in input of truncate: %s\n",s_f_inputLine);
		return ;
	}
	
	/* check formula and symbol */
	
	if ((formulaNum = FormulaNumber(formulaString)) >= s_f_fNum )
	{
		s_printf("Error in truncate: %s is not defined\n",formulaString);
		return ;
	}
	if ((symbolNum = SymbolNumber(symbolString)) >= s_f_sNum )
	{
		s_printf("Error in truncate: %s is not defined\n",symbolString);
		return ;
	}
		
	if (!s_f_formulas[formulaNum].sorted)
		SortFracNumFormula(formulaNum);
	
	if (!s_f_formulas[formulaNum].frac) /* polynomial */
	{
		TruncateFB(&s_f_formulas[formulaNum].fb, symbolNum, power);
	}
	else /* fractional */
	{
		if ( DecomposeFrac(&s_f_formulas[formulaNum].fb, &numFB, &denomFB) )
			return;

		TruncateFB(&numFB, symbolNum, power);
		TruncateFB(&denomFB, symbolNum, power);
		s_f_formulas[formulaNum].fb = ComposeFrac(numFB, denomFB);
	}

	PrintNumFormula(formulaNum);
}

int TruncateCheckInput(int inputNum, char *formulaString,
							char *symbolString, int power)
{
	if (inputNum !=3) return 1;
	if (power < 0) return 1;
	return 0;
}

void TruncateFB(s_f_FB **inFBP, int symbolNum, int power)
{
s_f_FB *curFB;

	TruncateTopFB(inFBP, symbolNum, power); /*truncating top is complicated */
	
	curFB = inFBP[0]->next;
	
	while (curFB)
	{
#if DEBUG_FORMULA
if (curFB->next)
{
	if (curFB->next->prev!=curFB)
	{
		printf("dif!\n");
		df(curFB,"curFB");
		df(curFB->next->prev,"curFB->next->prev");
	}
}
#endif

		if (curFB->base->power[symbolNum] > power)
		{
			DeleteOneFB(&curFB); /*curFB = curFB->next*/
		}
		else 
			curFB = curFB->next;
	}

}

void TruncateTopFB(s_f_FB **fbp, int symbolNum, int power)
{
s_f_FB *curFB;

	curFB = *fbp;
	
	while (curFB && curFB->base->power[symbolNum] > power)
#if TEST_DIFF
		FreeOneFB(&curFB);
#else
		curFB = curFB->next;
#endif	
	if (!curFB)
		curFB = MakeFB(0);
	
	*fbp = curFB;	
}


/**************************** Sum Truncate ***********************************/

void SumTruncateFormula(void)
{
int formulaNum;
int symbolSum;
int *symbolNumArray;
int power;
s_f_FB *numFB, *denomFB;

	if (!(symbolNumArray = calloc(s_f_symbolMax+1, sizeof(int))))
		s_OOMError();

	if (GetSumTruncParams(s_f_str, &formulaNum, &symbolSum, symbolNumArray, &power))
		return; /* error */
	
	if (!s_f_formulas[formulaNum].sorted)
		SortFracNumFormula(formulaNum);
	
	if (!s_f_formulas[formulaNum].frac) /* polynomial */
	{
		SumTruncateFB(&s_f_formulas[formulaNum].fb, symbolSum, symbolNumArray, power);
	}
	else /* fractional */
	{
		if ( DecomposeFrac(&s_f_formulas[formulaNum].fb, &numFB, &denomFB) )
			return;
		
		SumTruncateFB(&numFB, symbolSum, symbolNumArray, power);
		SumTruncateFB(&denomFB, symbolSum, symbolNumArray, power);
		s_f_formulas[formulaNum].fb = ComposeFrac(numFB, denomFB);
	}

	PrintNumFormula(formulaNum);
	
	free(symbolNumArray);
}

int GetSumTruncParams(char *inputline, int *formulaNumP,
						int *symbolSumP, int *symbolNumArray, int *totalPowerP)
{
char *curInput;
int symbolNum = 0;
char nameString[S_F_MAX_NAME_LENGTH+1];


	curInput = inputline;

	SkipSpaces(&curInput);


	if (!SScanString(&curInput, nameString))
		goto error;
	
	*formulaNumP = FormulaNumber(nameString);
	if (*formulaNumP >= s_f_fNum)
		goto error;
	
	while (*curInput && symbolNum <= s_f_symbolMax)
	{
		if (!SScanString(&curInput, nameString))
			goto error;
		if (isdigit(nameString[0])) /* total power */
		{
			*totalPowerP = atoi(nameString);
			*symbolSumP = symbolNum;
			return 0; /* no error */
		}
		else /* symbol name */
		{
			symbolNumArray[symbolNum] = SymbolNumber(nameString);
			if (symbolNumArray[symbolNum] >= s_f_sNum)
			{
				s_printf("%s is not defined.\n",nameString);
				goto error;
			}
			symbolNum++;
		}
	}
	
error:	printf("In GetSumTruncParams:\nError in %s\n",s_f_inputLine);
		return -1; /* error */
}

void SumTruncateFB(s_f_FB **inFBP, int symbolSum, int *symbolNumArray, int power)
{
s_f_FB *curFB;

	SumTruncateTopFB(inFBP, symbolSum, symbolNumArray, power); /*truncating top is complicated */
	
	curFB = inFBP[0]->next;
	
	while (curFB)
	{
#if DEBUG_FORMULA
if (curFB->next)
{
	if (curFB->next->prev!=curFB)
	{
		printf("dif!\n");
		df(curFB,"curFB");
		df(curFB->next->prev,"curFB->next->prev");
	}
}
#endif
		if (SumPower(curFB->base->power, symbolSum, symbolNumArray) > power)
		{
			DeleteOneFB(&curFB); /*curFB = curFB->next*/
		}
		else 
			curFB = curFB->next;
	}

}

void SumTruncateTopFB(s_f_FB **fbp, int symbolSum, int *symbolNumArray, int power)
{
s_f_FB *curFB;

	curFB = *fbp;
	
	while (SumPower(curFB->base->power, symbolSum, symbolNumArray) > power && curFB)
		curFB = curFB->next;
	
	if (!curFB)
		curFB = MakeFB(0);
	
	*fbp = curFB;	
}

int SumPower(int *powerArray, int symbolSum, int *symbolNumArray)
{
int sum = 0, sIndex;

	for ( sIndex = 0; sIndex < symbolSum; sIndex++)
		sum += powerArray[symbolNumArray[sIndex]];
	return sum;
}


/**************************** Expand Denominator ******************************/

void ExpandDenominator(void)
{
int power;
int formulaNum;
int symbolNum;
s_f_FB *numFB, *denomFB;

s_f_FB *dummyLastFB, *dummyDenomFB;

	if (GetParamsFNumSNumInt(s_f_str, &formulaNum, &symbolNum, &power))
		return; /* error */

	if (!s_f_formulas[formulaNum].sorted)
		SortFracNumFormula(formulaNum);

	if (!s_f_formulas[formulaNum].frac) /* polynomial */
	{
		s_printf("Formula %s is not fractional\n", s_f_fName[formulaNum]);
		return ;
	}

	if ( DecomposeFrac(&s_f_formulas[formulaNum].fb, &numFB, &denomFB) )
		return;

	TruncateFB(&numFB, symbolNum, power);
	TruncateFB(&denomFB, symbolNum, power);

	EDFrac(&numFB, &denomFB, symbolNum, power);
#if 1
	ExpandFB(&numFB, &dummyLastFB, &dummyDenomFB);
	ExpandFB(&denomFB, &dummyLastFB, &dummyDenomFB);

	TruncateFB(&numFB, symbolNum, power);
#endif

	s_f_formulas[formulaNum].fb = ComposeTwoFB(numFB, denomFB, DivOp);
	s_f_formulas[formulaNum].divDepth = 1;
	s_f_formulas[formulaNum].expanded = 0;
	s_f_formulas[formulaNum].sorted = 0;
	s_f_formulas[formulaNum].frac = 1;
	PrintNumFormula(formulaNum);
}

void EDFrac(s_f_FB **numFBP, s_f_FB **denomFBP, int symbolNum, int power)
{
s_f_FB *denom0, *denom1;
s_f_FB *newNumFB, *num2FB;


	if (DivideDenom(*denomFBP, symbolNum, &denom0, &denom1))
		return; /*error*/

	num2FB = MakeNum2(denom0, denom1, symbolNum, power);

	newNumFB = ComposeTwoFB(*numFBP, num2FB, MulOp);
	denom0->power = power+1;

	*numFBP = newNumFB;
	*denomFBP = denom0;

	return ;
}

int DivideDenom(s_f_FB *denomFB, int symbolNum, s_f_FB **retDenom0FBP, s_f_FB **retDenom1FBP)
{
s_f_FB *denom0 = 0, *curDenom0 = 0, *curInDenom;
s_f_FB *denom1 = 0, *curDenom1 = 0;

	curInDenom = denomFB;
	
	while (curInDenom)
	{
		if (curInDenom->base->power[symbolNum])
		{
			if (curDenom1)
			{
				curDenom1 = AppendNextOneFB(curDenom1, curInDenom);
			}
			else
			{
				denom1 = CopyOneFB(curInDenom);
				curDenom1 = denom1;
			}
		}
		else
		{
			if (curDenom0)
			{
				curDenom0 = AppendNextOneFB(curDenom0, curInDenom);
			}
			else
			{
				denom0 = CopyOneFB(curInDenom);
				curDenom0 = denom0;
			}
		}
		curInDenom = curInDenom->next;
	}

	

	if (!denom0)
	{
		s_printf("Error in DivideDenom:\n0-th order is empty.\n");
		return -1; /* error */	
	}
	
	if (!denom1)
	{
		denom1 = MakeFB(0);	
	}

	*retDenom0FBP = denom0;
	*retDenom1FBP = denom1;

	return 0;
}

s_f_FB *MakeNum2(s_f_FB *denom0, s_f_FB *denom1, int symbolNum, int power)
{
s_f_FB *newNum2FB, *curNewFB;

	curNewFB = newNum2FB = NewFormulaBlock();
	newNum2FB->type = BlockType;
	newNum2FB->operand = AddOp;
	newNum2FB->block = CopyFB(denom0);
	newNum2FB->block->power = power;
		
	
//	newNum2FB = EDMakeSeries(newNum2FB, denom0, denom1, power); /* return last */
	EDMakeSeries(newNum2FB, denom0, denom1, power, symbolNum); /* return last */
	
	return newNum2FB;
}

void EDMakeSeries(s_f_FB *curNewFB, s_f_FB *denom0, s_f_FB *denom1, int power, int symbolNum)
{
s_f_FB *denom0PowerFB, *denom1PowerFB, *topFB;
int subPower;

	topFB = curNewFB;

	for (subPower = 1; subPower <= power; subPower++)
	{
		curNewFB = MakeNextBlock(curNewFB);

		if (subPower & 1)
			curNewFB->operand = SubOp;
		else
			curNewFB->operand = AddOp;
		
			
		denom1PowerFB = CopyFB(denom1);
		
		TruncateFB(&denom1PowerFB, symbolNum, power + 1 - subPower);
		
			
		
		denom1PowerFB->power = subPower;
		
		if (subPower < power)  /* multiply denom0 */
		{
			denom0PowerFB = CopyFB(denom0);
			denom0PowerFB->power = power - subPower;
			
			curNewFB->type = BlockType;
			curNewFB->block = ComposeTwoFB(denom0PowerFB, denom1PowerFB, MulOp);
		}
		else
		{
			curNewFB->type = BlockType;
			curNewFB->block = denom1PowerFB;
		}

	}
	
	return;
}	


/**************************** Reduce Fraction ******************************/

void ReduceFormula(void)
{
int formulaNum;
int symbolNum;
int power;
s_f_FB *numFB, *denomFB, *curFB;
int coefFlag = 0;
double coef = 1;
int fracFlag;

	if (GetReduceParams(s_f_str, &formulaNum, &symbolNum, &power, &coefFlag, &coef))
		return; /* error */
	
	if (!s_f_formulas[formulaNum].sorted)
		SortFracNumFormula(formulaNum);
	
	fracFlag = s_f_formulas[formulaNum].frac;

	if (fracFlag) 
	{
		if ( DecomposeFrac(&s_f_formulas[formulaNum].fb, &numFB, &denomFB) )
			return;
	}
	else /* polynomial */
	{
		numFB = s_f_formulas[formulaNum].fb;
	}


	if (coefFlag) /* reduce coefficient */
	{
		if (ReduceCoefFB(&numFB, coef))
		{
			goto error;
		}

		if (fracFlag)
			if (ReduceCoefFB(&denomFB, coef))
			{
				goto error;
			}
	}
	else /* reduce symbol power */
	{
		if (ReduceFB(&numFB, symbolNum, power, &curFB))
		{
			UndoReduceFB(&numFB,  symbolNum, power, curFB);
			goto error;
		}
		if (fracFlag)
			if (ReduceFB(&denomFB, symbolNum, power, &curFB))
			{
				UndoReduceFB(&numFB,  symbolNum, power, 0);
				UndoReduceFB(&denomFB,  symbolNum, power, curFB);
				goto error;
			}
	}

	if (fracFlag)
	{
		s_f_formulas[formulaNum].fb = ComposeFrac(numFB, denomFB);
	}
	else
	{
		s_f_formulas[formulaNum].fb = numFB;
	}
	
	PrintNumFormula(formulaNum);
	return;

error:
	s_printf("Can not reduce by %s^%d\n",
					s_f_symbolTable[symbolNum].printName, power);
	return;
}

int GetReduceParams(char *inputline, int *formulaNumP,
			int *symbolNumP, int *powerP,
			int *coefFlagP, double *coefP)
{
char *curInput;
char nameString[S_F_MAX_NAME_LENGTH+1];

	curInput = inputline;

	SkipSpaces(&curInput);

	if (!SScanString(&curInput, nameString))
		goto error;	
	*formulaNumP = FormulaNumber(nameString);
	if ((*formulaNumP) >= s_f_fNum)
		goto error;
	
	if (!SScanString(&curInput, nameString))
		goto error;

	if (isdigit(nameString[0]) || nameString[0] == '.' || nameString[0] == '-') /* real number */
	{
		*coefFlagP = 1;
		*coefP = atof(nameString);
		return s_NoErr; /*** is reduce coefficient ***/
	}
	else
		*coefFlagP = 0;

	*symbolNumP = SymbolNumber(nameString);
	if ((*symbolNumP) >= s_f_sNum)
	{
		s_printf("%s is not defined.\n",nameString);
		goto error;
	}

	if (!SScanInt(&curInput, powerP))
		*powerP = 1;
	
	return s_NoErr;

error:
	printf("In GetReduceParams:\nError in %s\n",s_f_inputLine);
	return -1; /* error */
}

int ReduceFB(s_f_FB **inFBP, int symbolNum, int power, s_f_FB **curFBP)
{
s_f_FB *curFB;

	curFB = *inFBP;

	while (curFB)
	{
		if (curFB->base->power[symbolNum] < power)
		{
			*curFBP = curFB;
			return -1; /* error */
		}
		curFB->base->power[symbolNum] -= power;
		curFB = curFB->next;
	}
	
	return s_NoErr;
}

void UndoReduceFB(s_f_FB **inFBP, int symbolNum, int power, s_f_FB *lastFB)
{
s_f_FB *curFB;

	curFB = *inFBP;

	while (curFB && curFB!=lastFB)
	{
		curFB->base->power[symbolNum] += power;
		curFB = curFB->next;
	}
	
	return;
}

int ReduceCoefFB(s_f_FB **inFBP, double coef)
{
s_f_FB *curFB;

	curFB = *inFBP;

	while (curFB)
	{
		curFB->base->coef /= coef;
		curFB = curFB->next;
	}
	
	return s_NoErr;
}

/**************************** Select Powers ******************************/

/* SelectFormula(inputline) = SelectCoefFormula(inputline,0) */

void SelectCoefFormula(int coefFlag)
{
int formulaNum;
int symbolNum;
int power;
s_f_FB *numFB, *denomFB;

	if (GetParamsFNumSNumInt(s_f_str, &formulaNum, &symbolNum, &power))
		return; /* error */
	
	if (!s_f_formulas[formulaNum].sorted)
		SortFracNumFormula(formulaNum);
	
	if (!s_f_formulas[formulaNum].frac) /* polynomial */
	{
		SelectFB(&s_f_formulas[formulaNum].fb, symbolNum, power, coefFlag);
	}
	else /* fractional */
	{
		if ( DecomposeFrac(&s_f_formulas[formulaNum].fb, &numFB, &denomFB) )
			return;
		
		SelectFB(&numFB, symbolNum, power, coefFlag);
		s_f_formulas[formulaNum].fb = ComposeFrac(numFB, denomFB);
	}

	PrintNumFormula(formulaNum);
}

int SelectFB(s_f_FB **inFBP, int symbolNum, int power, int coefFlag)
{
s_f_FB *curFB;

	SelectTopFB(inFBP, symbolNum, power, coefFlag);
	
	curFB = inFBP[0]->next;
	
	while (curFB)
	{
#if DEBUG_FORMULA
if (curFB->next)
{
	if (curFB->next->prev!=curFB)
	{
		printf("dif!\n");
		df(curFB,"curFB");
		df(curFB->next->prev,"curFB->next->prev");
	}
}
#endif
		if (curFB->base->power[symbolNum] != power)
		{
			DeleteOneFB(&curFB); /*curFB = curFB->next*/
		}
		else 
		{
			if (coefFlag)
				curFB->base->power[symbolNum] = 0;
			curFB = curFB->next;
		}
	}
	return s_NoErr;
}

void SelectTopFB(s_f_FB **fbp, int symbolNum, int power, int coefFlag)
{
s_f_FB *curFB;

	curFB = *fbp;
	
	while (curFB && curFB->base->power[symbolNum] != power)
		curFB = curFB->next;
	
	if (!curFB)
		curFB = MakeFB(0);
	else
	{
		curFB->prev = 0;
		
		if (coefFlag)
			curFB->base->power[symbolNum] = 0;
	}	
	
	*fbp = curFB;	
}


/**************************** Get Numerator and Denominator ******************************/

void GetNumerator(void)
{
int formulaNum;
s_f_FB *numFB, *denomFB;

	if (GetParamsFNum(s_f_str, &formulaNum))
		return; /* error */

	if (!s_f_formulas[formulaNum].sorted)
		SortFracNumFormula(formulaNum);
	
	if (!s_f_formulas[formulaNum].frac) /* polynomial */
	{
		s_printf("Formula %s is not fractional\n", s_f_fName[formulaNum]);
		return ;
	}

	
	if ( DecomposeFrac(&s_f_formulas[formulaNum].fb, &numFB, &denomFB) )
		return;
	
	s_f_formulas[formulaNum].fb = numFB;
	s_f_formulas[formulaNum].frac = 0;
	
	FreeBaseFBs(denomFB);

	PrintNumFormula(formulaNum);
}

void GetDenominator(void)
{
int formulaNum;
s_f_FB *numFB, *denomFB;

	if (GetParamsFNum(s_f_str, &formulaNum))
		return; /* error */
	
	if (!s_f_formulas[formulaNum].sorted)
		SortFracNumFormula(formulaNum);
	
	if (!s_f_formulas[formulaNum].frac) /* polynomial */
	{
		s_printf("Formula %s is not fractional\n", s_f_fName[formulaNum]);
		return ;
	}

	
	if (DecomposeFrac(&s_f_formulas[formulaNum].fb, &numFB, &denomFB))
		return;
	
	s_f_formulas[formulaNum].fb = denomFB;
	s_f_formulas[formulaNum].frac = 0;

	FreeBaseFBs(numFB);

	PrintNumFormula(formulaNum);
}


/**************************** Set Compare Order ***********************************/

void SetCompareOrder(void)
{
int symbolSum;
int *symbolNumArray;
int i, j;


	if (!(symbolNumArray = calloc(s_f_symbolMax+1, sizeof(int))))
		s_OOMError();

	SkipSpaces(&s_f_str);

	if (!s_f_str[0])
	{
		s_f_compareBasesP = &CompareBases;
		return;
	}

	if (GetSetCompareOrderParams(s_f_str, &symbolSum, symbolNumArray))
		return; /* error */
	
	for (i = 0; i < symbolSum; i++)
	{
		s_f_compareOrder[i] = symbolNumArray[i];
	}

	i = symbolSum;
	for (j = 0; j < s_f_sNum; j++)
	{
		if ( ! InSet(j, symbolSum, symbolNumArray))
		{
			s_f_compareOrder[i] = j;
			i++;
		}
	}

	s_f_compareBasesP = &OrderCompareBases;
	
	for (i = 0; i < s_f_fNum; i++)
		s_f_formulas[i].sorted = 0;
		
	free(symbolNumArray);
}

int GetSetCompareOrderParams(char *inputline,
						int *symbolSumP, int *symbolNumArray)
{
char *curInput;
int symbolNum = 0;
char nameString[S_F_MAX_NAME_LENGTH+1];


	curInput = inputline;

	SkipSpaces(&curInput);


	while (*curInput && symbolNum < s_f_symbolMax)
	{
		if (!SScanString(&curInput, nameString))
			return -1;

		symbolNumArray[symbolNum] = SymbolNumber(nameString);
		if (symbolNumArray[symbolNum] >= s_f_sNum)
		{
			s_printf("%s is not defined.\n",nameString);
			goto error;
		}
		symbolNum++;
	}
	
	*symbolSumP = symbolNum;
	return s_NoErr;

error:	printf("In GetSetCompareOrderParams:\nError in %s\n",s_f_inputLine);
		return -1; /* error */
}


/**************************** Substitute ***********************************/

void SubstituteFormula(void)
{
int formulaNum, symbolNum, subsFNum;

	if (GetParamsFNumSFNum(s_f_str, &formulaNum, &symbolNum, &subsFNum))
		return; /* error */
	
	if (formulaNum == subsFNum)
	{
		s_printf("Error in Substitute Formula:\nformula and symbol identical\n");
		return;
	}
	
	SubsFB(&(s_f_formulas[formulaNum].fb), symbolNum, subsFNum);

	s_f_formulas[formulaNum].expanded = 0;
	s_f_formulas[formulaNum].sorted = 0;
	
	if (s_f_formulas[subsFNum].frac)
		s_f_formulas[formulaNum].frac = 1;
	
	PrintNumFormula(formulaNum);
}

void SubsFB(s_f_FB **inFB, int symbolNum,int subsFNum)
{
s_f_FB *curFB, *baseFB, *subsFB;

	curFB = *inFB;
	
	while(curFB)
	{
		if (curFB->type == BlockType)
		{
			SubsFB(&(curFB->block), symbolNum, subsFNum);
		}
		else /* BaseType */
		{
			if (curFB->base->power[symbolNum] )
			{
				baseFB = curFB->block = NewFormulaBlock();
				baseFB->type = BaseType;
				baseFB->operand = MulOp;
				baseFB->base = curFB->base;
				
				subsFB = MakeNextBlock(baseFB);
				subsFB->type = BlockType;
				subsFB->operand = MulOp;
				subsFB->block = CopyFB(s_f_formulas[subsFNum].fb);
				subsFB->block->power *= curFB->base->power[symbolNum];
				
				baseFB->base->power[symbolNum] = 0;
				curFB->type = BlockType;
				curFB->base = 0;
			}
		}
		curFB = curFB->next;
	}
}




#if TEST_CO
/**************************** Copy Formula ******************************/

void CopyFormula(void)
{
int formula1Num, formula2Num;

	if (GetParamsFNum(s_f_str, &formula1Num))
		return; /* error */

	ReadAndAddFormula(&formula2Num);
	
	s_f_formulas[formula2Num].divDepth = s_f_formulas[formula1Num].divDepth;
	s_f_formulas[formula2Num].expanded = s_f_formulas[formula1Num].expanded;
	s_f_formulas[formula2Num].sorted = s_f_formulas[formula1Num].sorted;
	s_f_formulas[formula2Num].frac = s_f_formulas[formula1Num].frac;
	s_f_formulas[formula2Num].controlFlag = s_f_formulas[formula1Num].controlFlag;
	
	s_f_formulas[formula2Num].fb = CopyFB(s_f_formulas[formula1Num].fb);

	PrintNumFormula(formula2Num);
	
	return;
}

void ReadAndAddFormula(int *fNumP)
{
char fString[S_F_MAX_NAME_LENGTH+1];
char printFName[S_F_MAX_NAME_LENGTH+1];
s_f_ControlPB fControlPB;

	if ( ReadControlOrSymbolName(fString, printFName, &fControlPB) )
		return;

	if (AddFormula(fString, fNumP, printFName, fControlPB.usedFlag) )
		return;
}

#endif /*TEST_CO*/


#if TEST_SOL
/**************************** Collect Formula ******************************/

void CollectFormula(void)
{
s_f_FB *numFB, *denomFB;
int formulaNum, symbolNum;

	if (GetParamsFNumSNum(s_f_str, &formulaNum, &symbolNum))
		return; /* error */
	
	if (!s_f_formulas[formulaNum].sorted)
		SortFracNumFormula(formulaNum);

	if (s_f_formulas[formulaNum].frac)
	{
		if (DecomposeFrac(&(s_f_formulas[formulaNum].fb), &numFB, &denomFB))
			return ; /* void func does not return error value */
		
		if (CollectFB(&numFB, symbolNum)) return;
		if (CollectFB(&denomFB, symbolNum)) return;
		
		s_f_formulas[formulaNum].fb = ComposeTwoFB(numFB, denomFB, DivOp);
	}
	else
	{
		if (CollectFB(&(s_f_formulas[formulaNum].fb), symbolNum)) return;
	}
	
	s_f_formulas[formulaNum].expanded = 0;
	s_f_formulas[formulaNum].sorted = 0;	

	PrintNumFormula(formulaNum);
}

int CollectFB(s_f_FB **retFBP, int symbolNum)
{
s_f_FB **collectFBArray;
s_f_FB *retFB, *curFB, *powerFB;
int maxPower, power;
int err;

	ErrTest(GetCollectArrayFB(*retFBP, &collectFBArray, symbolNum, &maxPower));
	
	retFB = NewFormulaBlock();
	/*retFB->type = BlockType;*/  /* no need */
	
	powerFB = NewBaseFB();
	powerFB->base->coef = 1;
	powerFB->operand = AddOp;
	
	retFB->block = ComposeTwoFB(powerFB, collectFBArray[0], MulOp);
	
	curFB = retFB;
	
	for (power = 1; power <= maxPower; power++)
	{
		curFB = MakeNextBlock(curFB);
		
		powerFB = CopyOneFB(powerFB);
		(powerFB->base->power[symbolNum])++;
		curFB->block = ComposeTwoFB(powerFB, collectFBArray[power], MulOp);
	}
	
	*retFBP = retFB;
	return s_NoErr;
}

int GetCollectArrayFB(s_f_FB *inFB, s_f_FB ***collectFBArrayP,
						int symbolNum, int *maxPowerP)
{
int maxPower, power;
s_f_FB *curFB;
int err;
s_f_FB **collectFBArray;
s_f_FB **curArray;

	ErrTest(MaxPowerFB(inFB, symbolNum, &maxPower));
	
	if (!(collectFBArray = calloc(maxPower+1, sizeof(s_f_FB*))))
		s_OOMError();
	if (!(curArray = calloc(maxPower+1, sizeof(s_f_FB*))))
		s_OOMError();
		
	curFB = inFB;
	
	while (curFB)
	{
		if (curFB->type != BaseType)
			return -1; /*error*/
		
		power = curFB->base->power[symbolNum];
		
		if ( curArray[power])
		{
			curArray[power] = AppendNextOneFB(curArray[power], curFB);
		}
		else
		{
			curArray[power] = collectFBArray[power] = CopyOneFB(curFB);
		}
		curArray[power]->base->power[symbolNum] = 0;
		curFB = curFB->next;
	}
	
	for (power = 0; power <= maxPower; power++)
	{
		if (! collectFBArray[power])
			collectFBArray[power] = MakeFB(0);
	}
	
	*collectFBArrayP = collectFBArray;
	*maxPowerP = maxPower;
	
	return s_NoErr;
}


int MaxPowerFB(s_f_FB *inFB, int symbolNum, int *maxPowerP)
{
int maxPower = 0;
s_f_FB *curFB;

	curFB = inFB;
	
	while (curFB)
	{
		if (curFB->type != BaseType)
		{
			s_printf("Error at maxPowerFB:\nnot BaseType\n");
			return -1; /*error*/
		}
		
		maxPower = max(maxPower, curFB->base->power[symbolNum]);

		curFB = curFB->next;
	}
	*maxPowerP = maxPower;
	
	return s_NoErr;
}

/**************************** Solve Formula ******************************/


void SolveFormula(void)
{
s_f_FB *numFB, *denomFB;
int maxPower;
s_f_FB **collectArray;
int formulaNum, symbolNum, newFNum;

	if (GetParamsFNumSNum(s_f_str, &formulaNum, &symbolNum))
		return; /* error */
	
	if (!s_f_formulas[formulaNum].sorted)
		SortFracNumFormula(formulaNum);

	if (s_f_formulas[formulaNum].frac)
	{
		if (DecomposeFrac(&(s_f_formulas[formulaNum].fb), &numFB, &denomFB))
			return ; /* void func does not return error value */
	}
	else
	{
		numFB = s_f_formulas[formulaNum].fb;
	}
	
	if (MaxPowerFB(numFB, symbolNum, &maxPower)) return;
	
	if (maxPower != 1)
	{
		s_printf("Error at SolveFormula:\nPower not one\n");
		return ;
	}
	
	if (GetCollectArrayFB(numFB, &collectArray, symbolNum, &maxPower)) return;
	
	InvertSigns(&(collectArray[0]));
	
	AddFormula(s_f_sName[symbolNum], &newFNum, 
				s_f_symbolTable[symbolNum].printName, s_f_symbolTable[symbolNum].controlFlag);
	
	s_f_formulas[newFNum].expanded = 1;
	s_f_formulas[newFNum].sorted = 1;
	s_f_formulas[newFNum].divDepth = 1;
	s_f_formulas[newFNum].frac = 1;
	s_f_formulas[newFNum].fb = ComposeTwoFB(collectArray[0], collectArray[1], DivOp);
	
	PrintNumFormula(newFNum);
	
}

#endif /*TEST_SOL*/

#if TEST_DIV
/**************************** Polynomial Division ******************************/

void DivideFormula(void)
{
	DivideOrFactorFormula(0);
}

void FactorFormula(void)
{
	DivideOrFactorFormula(1);
}

void DivideOrFactorFormula(int factorFlag)
{
int fNum, redFNum;
s_f_FB *termFB, *divFB, *redFB;
char redName[S_F_MAX_NAME_LENGTH+1];
char redPrintName[S_F_MAX_NAME_LENGTH+1] = "";
int sNum, power;
s_f_FB *quotFB, *mulFB, *subFB;


	if (GetParamsFnumTermFBS(&fNum, &termFB, &divFB, &sNum, &power))
		return; /* error */

	if (!s_f_formulas[fNum].sorted)
		SortFracNumFormula(fNum);

	if (s_f_formulas[fNum].frac)
	{
		s_printf("Error at DivideFormula:\nCan not divide fractioal formula\n");
		return ; /* void func does not return error value */
	}

	if (DivideFB(&(s_f_formulas[fNum].fb), sNum, power, divFB, &redFB)) return;
	
	if (factorFlag)
	{
		quotFB = s_f_formulas[fNum].fb;
		subFB = ComposeTwoFB(termFB, divFB, SubOp);
		ExpandAndSortFB(&subFB);
		mulFB = ComposeTwoFB(subFB, quotFB, MulOp);
		s_f_formulas[fNum].fb = ComposeTwoFB(mulFB, redFB, AddOp);
		s_f_formulas[fNum].expanded = 0;
		s_f_formulas[fNum].sorted = 0;
		PrintNumFormula(fNum);
	}
	else
	{
		PrintNumFormula(fNum);
		
		sprintf(redName,"%s_r",s_f_fName[fNum]);
		if (s_f_formulas[fNum].controlFlag)
			sprintf(redPrintName,"%s_r",s_f_formulas[fNum].printName);		
		AddFormula(redName, &redFNum, redPrintName, s_f_formulas[fNum].controlFlag);
		s_f_formulas[redFNum].fb = redFB;
		
		PrintNumFormula(redFNum);
	}
}

int GetParamsFnumTermFBS(int *fNumP, s_f_FB **termFBP, s_f_FB **divFBP, int *sNumP, int *powerP)
{
int err;
char nameString[S_F_MAX_NAME_LENGTH+1];
int divDepth;
s_f_FB *lastFB, *denomFB;
int maxPower;

	SkipSpacesAndReadCont();

	if (!SScanString(&s_f_str, nameString))
		goto error;	
	*fNumP = FormulaNumber(nameString);
	if ((*fNumP) >= s_f_fNum)
		goto error;

	*termFBP = NewFormulaBlock();
	
	ErrTest(ReadTerm(termFBP,&divDepth));
	
	ErrTest(ReadFormulaBlock(divFBP, &divDepth));
	
	ErrTest(CheckGetSNumPower(*termFBP, sNumP, powerP) );
	
	ExpandFB(divFBP, &lastFB, &denomFB);
	
	MaxPowerFB(*divFBP, *sNumP, &maxPower);
	
	if ( maxPower >= *powerP)
	{
		s_printf("Error at GetParamsFnumTermFBS:\nPower of latter part too large\n");
		return -1;
	}
		
	return s_NoErr;

error:
	printf("Error at GetParamsFnumTermFBS:\nError in %s\n",s_f_inputLine);
	return -1; /* error */
}

int CheckGetSNumPower(s_f_FB *termFB, int *sNumP, int *powerP)
{
int i = 0;
int *power;

	if (termFB->base->coef != 1)
	{
		s_printf("Error in CheckGetSNumPower:\nFirwst coeff is not 1\n");
		return -1;
	}
	
	power = termFB->base->power;
	
	while (!power[i])
		i++;

	if (i >= s_f_sNum)
	{
		s_printf("Error in CheckGetSNumPower:\nNo variable in Term\n");
		return -1;
	}	
	
	*sNumP = i;
	
	if (power[i]!=1) /*for the time beeing*/
	{
		s_printf("Error in CheckGetSNumPower:\npower is not 1\n");
		return -1;
	}		
	*powerP = power[i];
	
	i++;
	for(; i < s_f_sNum; i++)
		if (power[i])
		{
			s_printf("Error in CheckGetSNumPower:\nTwo variables in Term\n");
			return -1;
		}
	
	return s_NoErr;
}

int DivideFB( s_f_FB **inFBP, int sNum, int power, s_f_FB *divFB, s_f_FB **redFBP)
{
s_f_FB **collectArray;
int maxPower;
int i;
s_f_FB *tmpFB, *tmp2FB;
s_f_FB *retFB, *curFB, *powerFB;

	InvertSigns(&divFB);
	
	GetCollectArrayFB(*inFBP, &collectArray, sNum, &maxPower);

	for( i = maxPower; i > 0; i--)
	{
		tmpFB = ComposeCopyTwoFB(collectArray[i], divFB, MulOp);
		tmp2FB = CopyFB(collectArray[i-1]);
		collectArray[i-1] = ComposeTwoFB(tmp2FB, tmpFB, AddOp);
		ExpandAndSortFB(&(collectArray[i-1]));
	}
	
	*redFBP = collectArray[0];

	retFB = NewFormulaBlock();
	/*retFB->type = BlockType;*/  /* no need */
	
	powerFB = NewBaseFB();
	powerFB->base->coef = 1;
	powerFB->operand = AddOp;
	
	retFB->block = ComposeTwoFB(powerFB, collectArray[1], MulOp);
	
	curFB = retFB;
	
	for (power = 2; power <= maxPower; power++)
	{
		curFB = MakeNextBlock(curFB);
		
		powerFB = CopyOneFB(powerFB);
		(powerFB->base->power[sNum])++;
		curFB->block = ComposeTwoFB(powerFB, collectArray[power], MulOp);
	}
	
	ExpandAndSortFB(&retFB);
	*inFBP = retFB;

	InvertSigns(&divFB);

	return s_NoErr;
}

int ExpandAndSortFB(s_f_FB **fbp)
{
s_f_FB *lastFB, *denomFB;

	dwf("ExpandAndSortFB 0\n");
	ExpandFB(fbp, &lastFB, &denomFB);
	SortFormulaBlock(fbp);
	
	dwf("ExpandAndSortFB END\n");
	return s_NoErr;
}
#endif /*TEST_DIV*/

/**************************** Clean Complex ******************************/

int CleanComplexFB(s_f_FB **inFBP, int symbolNum)
{
s_f_FB *curFB;
int eachpowermod4;

	curFB = *inFBP;
	while (curFB)
	{
		eachpowermod4=(curFB->base->power[symbolNum])%4;
		switch (eachpowermod4)
		{
			case 0:			/*i^(4n)=0*/
			curFB->base->power[symbolNum] = 0;
			curFB = curFB->next;
			break;

			case 1:			/*i^(4n+1)=i*/
			curFB->base->power[symbolNum] = 1;
			curFB = curFB->next;
			break;

			case 2:			/*i^(4n+2)=-1*/
			curFB->base->power[symbolNum] = 0;
			curFB->base->coef = -(curFB->base->coef);
			curFB = curFB->next;
			break;

			case 3:			/*i^(4n+3)=-i*/
			curFB->base->power[symbolNum] = 1;
			curFB->base->coef = -(curFB->base->coef);
			curFB = curFB->next;
			break;
		}
	}
	return s_NoErr;
}

void CleanComplexFormula(void)
{
int formulaNum;
int symbolNum;
s_f_FB *numFB, *denomFB;

	symbolNum=SymbolNumber("\\i");
	if(symbolNum>=s_f_sNum)
	{
		s_printf("'i' is not defined\n");
		s_printf("In CleanComplexFormula:\nError in %s\n",s_f_inputLine);
		return;
	}
	
	if (GetParamsFNum(s_f_str, &formulaNum))
		return; /* error */
	
	if (!s_f_formulas[formulaNum].sorted)
		SortFracNumFormula(formulaNum);
	
	if (!s_f_formulas[formulaNum].frac) /* polynomial */
	{
		CleanComplexFB(&s_f_formulas[formulaNum].fb, symbolNum);
	}
	else /* fractional */
	{
		if ( DecomposeFrac(&s_f_formulas[formulaNum].fb, &numFB, &denomFB) )
			return;
		
		CleanComplexFB(&numFB, symbolNum);
		CleanComplexFB(&denomFB, symbolNum);

		
	}

//	s_f_formulas[formulaNum].sorted=0;

	SortFracNumFormula(formulaNum);
	s_f_formulas[formulaNum].cleanComplexFlag=1;

	PrintNumFormula(formulaNum);
	
}

void CompMakeDenomReal(void)
{
int formulaNum;
int symbolNum;
s_f_FB *numFB, *denomFB, *drealFB, *dimagFB, *dreal2FB, *dimag2FB, *nrealFB, *nimagFB,*cpxiFB;
s_f_FB *boxd1FB,*boxd2FB;
s_f_FB *boxn1FB,*boxn2FB,*boxn3FB,*boxn4FB,*boxn5FB,*boxn6FB,*boxn7FB;

	symbolNum=SymbolNumber("\\i");
	if(symbolNum>=s_f_sNum)
	{
		s_printf("'i' is not defined\n");
		s_printf("In CleanComplexFormula:\nError in %s\n",s_f_inputLine);
		return;
	}
	
	if (GetParamsFNum(s_f_str, &formulaNum))
		return; /* error */
	
	if (!s_f_formulas[formulaNum].sorted)
		SortFracNumFormula(formulaNum);
	
	if (!s_f_formulas[formulaNum].frac) /* polynomial */
	{
		if(!s_f_formulas[formulaNum].cleanComplexFlag)
		{
		CleanComplexFB(&s_f_formulas[formulaNum].fb, symbolNum);
		}
	}
	else /* fractional */
	{
		if ( DecomposeFrac(&s_f_formulas[formulaNum].fb, &numFB, &denomFB) )
			return;
		if(!s_f_formulas[formulaNum].cleanComplexFlag)
		{
			CleanComplexFB(&numFB, symbolNum);
			CleanComplexFB(&denomFB, symbolNum);
		}
		
		nrealFB=CopyFB(numFB);
		nimagFB=CopyFB(numFB);
		drealFB=CopyFB(denomFB);
		dimagFB=CopyFB(denomFB);
		cpxiFB=MakeFB(1);
		cpxiFB->base->power[symbolNum] = 1;
		SelectFB(&nrealFB, symbolNum, 0, 1);
		SelectFB(&nimagFB, symbolNum, 1, 1);
		SelectFB(&drealFB, symbolNum, 0, 1);
		SelectFB(&dimagFB, symbolNum, 1, 1);

		dreal2FB = CopyFB(drealFB);
		dimag2FB = CopyFB(dimagFB);
		
		boxd1FB=ComposeCopyTwoFB(drealFB,drealFB,MulOp);
		boxd2FB=ComposeCopyTwoFB(dimagFB,dimagFB,MulOp);
		denomFB=ComposeCopyTwoFB(boxd1FB,boxd2FB,AddOp);
		
		boxn1FB=ComposeCopyTwoFB(nrealFB,drealFB,MulOp);
		boxn2FB=ComposeCopyTwoFB(nimagFB,dimagFB,MulOp);
		boxn3FB=ComposeCopyTwoFB(boxn1FB,boxn2FB,AddOp);
		
		boxn4FB=ComposeCopyTwoFB(nimagFB,drealFB,MulOp);
		boxn5FB=ComposeCopyTwoFB(nrealFB,dimagFB,MulOp);
		boxn6FB=ComposeCopyTwoFB(boxn4FB,boxn5FB,SubOp);
		boxn7FB=ComposeCopyTwoFB(boxn6FB,cpxiFB,MulOp);

		numFB=ComposeCopyTwoFB(boxn3FB,boxn7FB,AddOp);
		s_f_formulas[formulaNum].fb = ComposeCopyTwoFB(numFB, denomFB, DivOp);
	}

	s_f_formulas[formulaNum].expanded = 0;
	SortFracNumFormula(formulaNum);
	s_f_formulas[formulaNum].cleanComplexFlag=1;

	PrintNumFormula(formulaNum);
	
}


/**************************** Differentiate ******************************/

#if TEST_DIFF
void DifferentiateFormula(void)
{
int formulaNum;
int symbolNum;
s_f_FB *numFB, *denomFB, *dNumFB, *dDenomFB, *num1FB, *num2FB;
int fracFlag;

	dwf("DifferentiateFormula 0\n");
	if (GetParamsFNumSNum(s_f_str, &formulaNum, &symbolNum))
		return; /* error */
	
	if (!s_f_formulas[formulaNum].sorted)
		SortFracNumFormula(formulaNum);
	
	fracFlag = s_f_formulas[formulaNum].frac;

	if (fracFlag) 
	{
		if ( DecomposeFrac(&s_f_formulas[formulaNum].fb, &numFB, &denomFB) )
			return;
	}
	else /* polynomial */
	{
		numFB = s_f_formulas[formulaNum].fb;
	}

	if (fracFlag)
	{
		dNumFB = CopyFB(numFB);
		dDenomFB = CopyFB(denomFB);
		
		if (DifferentiateFB(&dNumFB, symbolNum))
			goto error;
		if (DifferentiateFB(&dDenomFB, symbolNum))
			goto error;
		
		num1FB = ComposeCopyTwoFB(dNumFB, denomFB, MulOp);
		num2FB = ComposeCopyTwoFB(numFB, dDenomFB, MulOp);
		numFB = ComposeTwoFB(num1FB, num2FB, SubOp);
		ExpandAndSortFB(&numFB);
		
		denomFB = ComposeCopyTwoFB(denomFB, denomFB, MulOp);
		ExpandAndSortFB(&denomFB);		
	}
	else
	{
		DifferentiateFB(&numFB, symbolNum);
	}

	if (fracFlag)
	{
		s_f_formulas[formulaNum].fb = ComposeFrac(numFB, denomFB);
	}
	else
	{
		s_f_formulas[formulaNum].fb = numFB;
	}
	
	PrintNumFormula(formulaNum);
	dwf("DifferentiateFormula END\n");
	return;

error:
	s_printf("Error in DifferentiateFormula\n");
	dwf("DifferentiateFormula END\n");
	return;
}

int DifferentiateFB(s_f_FB **inFBP, int symbolNum)
{
s_f_FB *curFB;

	dwf("DifferentiateFB 0\n");
	DifferentiateTopFB(inFBP, symbolNum); /*differentiate top is complicated */

	curFB = inFBP[0]->next;

	while (curFB)
	{
		if (curFB->base->power[symbolNum])
		{
			curFB->base->coef *= curFB->base->power[symbolNum];
			curFB->base->power[symbolNum] --;
			curFB = curFB->next;
		}
		else
		{
			DeleteOneFB(&curFB);			
		}
	}
	
	dwf("DifferentiateFB END\n");
	return s_NoErr;
}

void DifferentiateTopFB(s_f_FB **fbp, int symbolNum)
{
s_f_FB *curFB;

	dwf("DifferentiateTopFB 0\n");
	curFB = *fbp;
	
	while (curFB && (!curFB->base->power[symbolNum]) )
		FreeOneFB(&curFB);
	
	if (curFB)
	{
		curFB->base->coef *= curFB->base->power[symbolNum];
		curFB->base->power[symbolNum] --;
	}
	else
		curFB = MakeFB(0);
	
	*fbp = curFB;	
	dwf("DifferentiateTopFB END\n");
}

#endif /*TEST_DIFF*/

#if TEST_PRED
void PolynomialReduceFormula(void)
{
int fNum;
s_f_FB *termFB, *divFB;
int sNum, power;
s_f_FB *numFB, *denomFB;
s_f_FB *redNumFB, *redDenomFB;

	if (GetParamsFnumTermFBS(&fNum, &termFB, &divFB, &sNum, &power)) /*power should be returned */
		return; /* error */

	if (!s_f_formulas[fNum].sorted)
		SortFracNumFormula(fNum);

	if (!s_f_formulas[fNum].frac)
	{
		s_printf("Error at PolynomialReduceFormula:\nCan not reduce polynomials\n");
		return ; /* void func does not return error value */
	}

	if ( DecomposeFrac(&s_f_formulas[fNum].fb, &numFB, &denomFB) )
			return;

df(numFB, "numFB");
df(denomFB, "denomFB");

df(divFB, "divFB");
	if (DivideFB(&numFB, sNum, power, divFB, &redNumFB)) return;
df(divFB, "divFB");
	if (DivideFB(&denomFB, sNum, power, divFB, &redDenomFB)) return;

df(numFB, "numFB");
df(denomFB, "denomFB");
df(redNumFB, "redNumFB");
df(redDenomFB, "redDenomFB");

	if (redNumFB->base->coef)
	{
		s_printf("Error in PolynomialReduceFormula:\nCannot divide!\n");
		return;
	}

	if (redDenomFB->base->coef)
	{
		s_printf("Error in PolynomialReduceFormula:\nCannot divide!\n");
		return;
	}

	s_f_formulas[fNum].fb = ComposeFrac(numFB, denomFB);
	PrintNumFormula(fNum);
		
}
#endif

#endif /*pragma once*/
