/*
 *  Formula-Utilities.c
 *  Version 1.8 / 000721
 *  by Satoshi Takahashi
 */

#ifndef FORMULA_UTILITIES_C /*pragma once*/
#define FORMULA_UTILITIES_C

/************************ Switches *******************/

#include <Formula-Switches.h>

/************************ Include Headers *******************/

#include <ctype.h>
#include <stdlib.h>
#include <string.h>

#include <Formula-Common.h>
#include <Formula.h>
#include <Formula-v.h>
#include <Formula-Utilities.h>

#include <s_Common.h>
#include <StartSCL.h>
#include <s_String.h>
#include <s_Profile.h>
#include <s_Debug.h>


/**************************** Utilities ***********************************/

/*------ Input ------*/

void SkipSpacesAndReadCont(void)
{

readAgain:;

	while (isspace(s_f_str[0]))
		s_f_str++;

	if (s_f_str == s_f_strEnd && s_f_contLineFlag)
	{
		ReadOneLine();
		goto readAgain;
	}
}

void ReadContIfNeed(void)
{
	if (s_f_str == s_f_strEnd && s_f_contLineFlag)
		ReadOneLine();
}

void SFStrInc(void)
{
	s_f_str++;
	if (s_f_str == s_f_strEnd && s_f_contLineFlag)
		ReadOneLine();
}

int IsNameStart(char c)
{
	return isalpha(c) || c == '\\';
}
	

/*------ Multiply ------*/

void MultiplyBases(s_f_Base * base1, s_f_Base * base2)
{
int i;

	base1->coef *= base2->coef;
	for (i = 0; i < s_f_sNum; i++)
	{
		base1->power[i] += base2->power[i];
	}
}

void MultiplyBases3(s_f_Base * base1, s_f_Base * base2, s_f_Base * base3)
{
int i;

	base1->coef = base2->coef * base3->coef;
	for (i = 0; i < s_f_sNum; i++)
	{
		base1->power[i] = base2->power[i] + base3->power[i];
	}
}

s_f_FB * MultiplyPlusBlocks(s_f_FB **block1, s_f_FB **block2,
		 s_f_FB ** lastFB)
{
	return MultiplySignedBlocks(block1, block2, lastFB, PlusSign);
}

s_f_FB * MultiplyMinusBlocks(s_f_FB **block1, s_f_FB **block2,
		 s_f_FB ** lastFB)
{
	return MultiplySignedBlocks(block1, block2, lastFB, MinusSign);
}

s_f_FB * MultiplySignedBlocks(s_f_FB **inFBP1, s_f_FB **inFBP2,
		 s_f_FB ** lastFB, int sign)  /* name should be changed */
{
s_f_FB *curInFB1, *inFB1, *outFB, *curInFB2, *curOutFB, *prevCurOutFB;
int minus;


	curInFB1 = inFB1 = *inFBP1;
	curInFB2 = *inFBP2;

	outFB = NewBaseFB();

	curOutFB = outFB;
	while(curInFB2)
	{
		curInFB1 = inFB1;
		while(curInFB1)
		{
			if ((curInFB2->type == BaseType)&&(curInFB1->type == BaseType))
				MultiplyBases3(curOutFB->base,curInFB2->base,curInFB1->base);
			else
/* to be considered */									; 
			minus = sign + (curInFB2->operand == SubOp) + (curInFB1->operand == SubOp);
			if (curOutFB->base->coef < 0)
			{
				minus++;
				curOutFB->base->coef = - curOutFB->base->coef;
			}
			if (minus%2)
				curOutFB->operand = SubOp;
			else
				curOutFB->operand = AddOp;
			curInFB1 = curInFB1->next;
			{
				prevCurOutFB = curOutFB;
				curOutFB = MakeNextBaseFB(curOutFB);
			}
		}
		curInFB2 = curInFB2->next;
	}
	prevCurOutFB->next = 0;
	FreeOneBaseFB(&curOutFB);

/*  multiplied result should be sorted */
  	SortFormulaBlock(&outFB);
  	prevCurOutFB = FindLastBlock(outFB);

	*lastFB = prevCurOutFB;
	return outFB;
}

s_f_FB* MultiplySignedBlockAndBase(s_f_FB** blockP,
			s_f_Base** baseP, s_f_FB** lastBlockP, int sign)
{
s_f_FB *fb, *cfb, *lastBlock, *mfb2, *mcfb2;
s_f_Base *base;
int minus;


	cfb = fb = *blockP;
	base = *baseP;

	mfb2 = NewBaseFB();
	mfb2->base->coef = 1;

	mcfb2 = mfb2;
	while(cfb)
	{
		MultiplyBases3(mcfb2->base,cfb->base,base);
		minus = sign + (cfb->operand == SubOp);
		if (mcfb2->base->coef < 0)
		{
			minus++;
			mcfb2->base->coef = - mcfb2->base->coef;
		}
		if (minus%2)
			mcfb2->operand = SubOp;
		else
			mcfb2->operand = AddOp;

		lastBlock = mcfb2;
		mcfb2 = MakeNextBaseFB(mcfb2);
		mcfb2->operand = AddOp;
		mcfb2->base->coef = 1;

		cfb = cfb->next;
	}
	lastBlock->next = 0;
	free(mcfb2->base);
	free(mcfb2);
	*lastBlockP = lastBlock;
	return mfb2;
}

void NextFB(s_f_FB** fbp)
{
	if(fbp[0]) fbp[0] = fbp[0]->next; /* fbp[0] == *fbp */
}
 
void AddBlocksWithLast(s_f_FB** newLastFBP,s_f_FB* curBlockFB,s_f_FB* curBlockLastFB)
{
	newLastFBP[0]->next = curBlockFB;
	//newLastFBP[0]->next->top = 0; /* old code */
	newLastFBP[0] = curBlockLastFB;
}

s_f_FB* ComposeFrac(s_f_FB* numFB,s_f_FB* denomFB)
{
s_f_FB *retFB;

	retFB = NewFormulaBlock();
	//retFB->top = 1; /* old code */
	retFB->type = BlockType;
	retFB->operand = MulOp;
	retFB->block = numFB;
	retFB->next = MakeNextBlock(retFB);
	retFB->next->type = BlockType;
	retFB->next->operand = DivOp;
	retFB->next->block = denomFB;
	return retFB;
}

void InvertSigns(s_f_FB **block)
{
s_f_FB *cfb;

	cfb = *block;
	while(cfb)
	{
		switch(cfb->operand)
		{
			case AddOp:
				cfb->operand = SubOp;
				break;
			case SubOp:
				cfb->operand = AddOp;
				break;
			default:
				PrintError("Error in inverting signs");
				break;
		}
		cfb = cfb->next;
	}
}

void FBSignOp(s_f_FB **retFBP)
{
s_f_FB *curFB;

	dwf("FBSignOp 0\n");
	curFB = *retFBP;
	
	while (curFB && curFB->base->coef == 0)
	{
		DeleteOneFB(&curFB);
	}

	if (!curFB)
	{
		*retFBP = MakeFB(0);
		dwf("FBSignOp END\n");
		return;
	}
	
	*retFBP = curFB;
	
	while(curFB)
	{
		if (curFB->base->coef == 0)
		{
			DeleteOneFB(&curFB);
		}
		else if (curFB->base->coef > 0)
		{
			curFB = curFB->next;
		}
		else /* curFB->base->coef < 0*/
		{
			curFB->base->coef = - curFB->base->coef;
			if (curFB->operand == AddOp)
			{
				curFB->operand = SubOp;
			}
			else
			{
				curFB->operand = AddOp;
			}
			curFB = curFB->next;
			
		}
	}
	dwf("FBSignOp END\n");
}


int CompareBases(int *power1, int *power2)
{
int i = 0;

	for(i = 0; i < s_f_sNum; i++)
	{
		if (power1[i] > power2[i])
			return 1;
		else if (power1[i] < power2[i])
			return -1;
	}
	return 0;
}


s_f_FB* FindLastBlock(s_f_FB* fb)
{
	while(fb->next) fb = fb->next;
	return fb;
}

/*------ Copy and Make -------*/

#if 1 /* save memory */

s_f_FB* CopyFB(s_f_FB* orgBlock)
{
s_f_FB  *curBlock, *copyBlock, *copyCurBlock;

	curBlock = orgBlock;
	copyCurBlock = copyBlock = CopyOneFB(orgBlock);
	
	curBlock = curBlock->next;
	
	while(curBlock)
	{
		copyCurBlock = AppendNextOneFB(copyCurBlock,curBlock);
		
		curBlock = curBlock->next;
	}
	return copyBlock;
}

#else /*1*/
s_f_FB* CopyFB(s_f_FB* orgBlock)
{
s_f_FB  *curBlock, *copyBlock, *copyCurBlock, *prevBlock = 0;

	curBlock = orgBlock;
	copyCurBlock = copyBlock = NewFormulaBlock();
	
	while(curBlock)
	{
		*copyCurBlock = *curBlock;
		copyCurBlock->prev = prevBlock;
		if (curBlock->type == BlockType)
		{
			copyCurBlock->block = CopyFB(curBlock->block);
		}
		else /* BaseType */
		{
			copyCurBlock->base = CopyBase(curBlock->base);
		}
		prevBlock = copyCurBlock;
		if (curBlock = curBlock->next)
			copyCurBlock = MakeNextBlock(copyCurBlock);
	}
	return copyBlock;
}
#endif /*else 1*/

s_f_FB * CopyOneFB(s_f_FB *orgFB)
{
 s_f_FB *copyFB;
 int i;
 int *orgPower, *copyPower;
 
	
	if (orgFB->type == BaseType)
	{

#if 1 /* save memory */

		copyFB = NewBaseFB();
		copyFB->type = orgFB->type;
		copyFB->operand = orgFB->operand;
		copyFB->power = orgFB->power;

		copyFB->base->coef = orgFB->base->coef;
		
		orgPower = orgFB->base->power;
		copyPower = copyFB->base->power;		
		for (i=0; i< s_f_sNum; i++)
			copyPower[i] = orgPower[i];

#else /*1*/

	 	copyFB = NewFormulaBlock();
		*copyFB = *orgFB;
		copyFB->base = CopyBase(orgFB->base);


#endif /*else 1*/

	}
	else /* BlockType */
	{
	 	copyFB = NewFormulaBlock();
		*copyFB = *orgFB;
		copyFB->block = CopyFB(orgFB->block);
	}
	
	copyFB->prev = 0;
	copyFB->next = 0;
	
	return copyFB;
}

s_f_Base *CopyBase(s_f_Base *orgBase)
{
s_f_Base *copyBase;
int i;

	copyBase = NewFormulaBase();

	copyBase->coef = orgBase->coef;
	for (i=0; i< s_f_sNum; i++)
		copyBase->power[i] = orgBase->power[i];

	return copyBase;
}

s_f_FB* NewFormulaBlock(void)
{
s_f_FB* fbp;

	if(!(fbp = malloc(sizeof(s_f_FB))))
		s_OOMError();
	fbp->type = BlockType;
	fbp->operand = AddOp;
	fbp->power = 1;
	fbp->base = 0;
	fbp->block = 0;
	fbp->next = 0;
	fbp->prev = 0;
	return fbp;
}

s_f_Base* NewFormulaBase(void)
{
s_f_Base *base;

			if(!(base = malloc(sizeof(s_f_Base))))
				s_OOMError();
			if(!(base->power = calloc(s_f_symbolMax, sizeof(int))))
				s_OOMError();
			base->coef = 0;
			return base;
}

void SetBlockType(s_f_FB* fbp)
{
	fbp->type = BlockType;
	fbp->base = 0;
	fbp->block = NewFormulaBlock();
}

void SetBaseType(s_f_FB* fbp)
{
	fbp->type = BaseType;
	fbp->block = 0;
	fbp->base = NewFormulaBase();
}



s_f_FB *NewBaseFB(void)
{
s_f_FB *retFB;
int i;

	if (s_f_freeFBHead != s_f_freeFBTail)
	{
		retFB = s_f_freeFBHead;
		s_f_freeFBHead = s_f_freeFBHead->next;
		
		retFB->base->coef = 0;
		for (i = 0; i<s_f_sNum; i++)
			retFB->base->power[i] = 0;
		retFB->next = 0;
		retFB->prev = 0;
	}
	else
	{
		retFB = NewFormulaBlock();
		SetBaseType(retFB);
	}

	retFB->operand = AddOp;
	return retFB;
}

s_f_FB* MakeNextBaseFB(s_f_FB *fbp)
{
	fbp->next  = NewBaseFB();
	fbp->next->prev = fbp;
	return fbp->next;
}

s_f_FB* MakeNextBlock(s_f_FB* fbp)
{
	fbp->next  = NewFormulaBlock();
	fbp->next->prev = fbp;
	return fbp->next;
}

s_f_FB *MakeFB(double coef)
{
s_f_FB *newFB;

	newFB = NewBaseFB();
	newFB->base->coef = coef;
	return newFB;
}

s_f_FB *AppendNextOneFB(s_f_FB *curFB, s_f_FB *appendFB)
{
	curFB->next = CopyOneFB(appendFB);
	curFB->next->prev = curFB;
	return curFB->next;
}

void CopyOneBlock(s_f_FB* curFB, s_f_FB** newTopFBP) /* old? */
{
	*newTopFBP = NewFormulaBlock();
	**newTopFBP = *curFB;
	newTopFBP[0]->next = 0;
	if (newTopFBP[0]->operand == MulOp)
		newTopFBP[0]->operand = AddOp;
}

void AppendOneBlock(s_f_FB* curFB, s_f_FB** newLastFBP) /* old? */
{
	CopyOneBlock(curFB, &(newLastFBP[0]->next));
	*newLastFBP = newLastFBP[0]->next;
}

/*------ Delete and Free -------*/

void DeleteOneFB(s_f_FB **delFBP) /*delFBP is substituted by delFBP->next*/
{
s_f_FB *fb;

	dwf("DeleteOneFB 0\n");
	fb = *delFBP;
	if (fb->prev)
		fb->prev->next = fb->next;
	if (fb->next)
		fb->next->prev = fb->prev;
	*delFBP = fb->next;

	if (s_f_GCFlag)
	{
		FreeOneBaseFB(&fb);
	}
	dwf("DeleteOneFB END\n");
}

void FreeOneBaseFB(s_f_FB **freeFBP)
{
s_f_FB *fb;

	fb = *freeFBP;
	if (!fb)
		PrintError("Error in FreeOneBaseFB:\n formula block empty\n");
	*freeFBP = fb->next;
	s_f_freeFBTail->next = fb;
	fb->prev = s_f_freeFBTail;
	s_f_freeFBTail = s_f_freeFBTail->next;
}

void FreeBaseFBs(s_f_FB *fb)
{
s_f_FB *lastFB;

	lastFB = FindLastBlock(fb);
	
	FreeOneBaseFB(&fb);
	
	s_f_freeFBTail = lastFB;
}

void FreeBaseFBsWithLast(s_f_FB *topFB, s_f_FB *lastFB)
{
	
	FreeOneBaseFB(&topFB);
	
	s_f_freeFBTail = lastFB;
}

void FreeOneFB(s_f_FB **freeFBP) /* slow */
{
s_f_FB *fb;

	fb = *freeFBP;
	if (fb->type == BaseType)
	{
		free(fb->base);
	}
	*freeFBP = fb->next;
	free(fb);
}

/*-------------------------- Get Parameters ---------------------------*/

int GetParamsFNum(char *inputline, int *formulaNumP)
{
char nameString[S_F_MAX_NAME_LENGTH+1];

	s_f_str = inputline;

	SkipSpaces(&s_f_str);

	/* if empty use current number */
	if (!SScanString(&s_f_str, nameString))
	{
		strcpy(nameString, "");
	}

	*formulaNumP = FormulaNumber(nameString);
	if ((*formulaNumP) >= s_f_fNum)
		goto error;
		
	return s_NoErr;

error:
	printf("Error at GetParamsFNum:\nError in %s\n",s_f_inputLine);
	return -1; /* error */
}

int GetParamsFNumSNum(char *inputline, int *formulaNumP,
			int *symbolNumP)
{
char nameString[S_F_MAX_NAME_LENGTH+1];

	s_f_str = inputline;

	SkipSpaces(&s_f_str);

	if (!SScanString(&s_f_str, nameString))
		goto error;	
	*formulaNumP = FormulaNumber(nameString);
	if ((*formulaNumP) >= s_f_fNum)
		goto error;
	
	if (!SScanString(&s_f_str, nameString))
		goto error;
	*symbolNumP = SymbolNumber(nameString);
	if ((*symbolNumP) >= s_f_sNum)
	{
		s_printf("%s is not defined.\n",nameString);
		goto error;
	}

	return s_NoErr;

error:
	printf("In GetParamsFNumSNumInt:\nError in %s\n",s_f_inputLine);
	return -1; /* error */
}

int GetParamsFNumSNumInt(char *inputline, int *formulaNumP,
			int *symbolNumP, int *powerP)
{
char nameString[S_F_MAX_NAME_LENGTH+1];

	s_f_str = inputline;

	SkipSpaces(&s_f_str);

	if (!SScanString(&s_f_str, nameString))
		goto error;	
	*formulaNumP = FormulaNumber(nameString);
	if ((*formulaNumP) >= s_f_fNum)
		goto error;
	
	if (!SScanString(&s_f_str, nameString))
		goto error;
	*symbolNumP = SymbolNumber(nameString);
	if ((*symbolNumP) >= s_f_sNum)
	{
		s_printf("%s is not defined.\n",nameString);
		goto error;
	}

	if (!SScanInt(&s_f_str, powerP))
		goto error;
	
	return s_NoErr;

error:
	printf("In GetParamsFNumSNumInt:\nError in %s\n",s_f_inputLine);
	return -1; /* error */
}


int GetParamsFNumSFNum(char *inputline, int *formulaNumP,
			int *symbolNumP, int *symbolFormulaNumP)
{
char nameString[S_F_MAX_NAME_LENGTH+1];

	s_f_str = inputline;

	SkipSpaces(&s_f_str);

	if (!SScanString(&s_f_str, nameString))
		goto error;	
	*formulaNumP = FormulaNumber(nameString);
	if ((*formulaNumP) >= s_f_fNum)
		goto error;
	
	if (!SScanString(&s_f_str, nameString))
		goto error;
	*symbolNumP = SymbolNumber(nameString);
	if ((*symbolNumP) >= s_f_sNum)
	{
		s_printf("%s is not used.\n",nameString);
		goto error;
	}

	*symbolFormulaNumP = FormulaNumber(nameString);
	if ((*symbolFormulaNumP) >= s_f_fNum)
	{
		s_printf("%s is not defined.\n",nameString);
		goto error;
	}

	return s_NoErr;

error:
	printf("In GetParamsFNumSNumInt:\nError in %s\n",s_f_inputLine);
	return -1; /* error */
}


/*-------------------------- Other ---------------------------*/

int SymbolNumber(char *sname)
{
int i;

	for(i=0;i<s_f_sNum;i++)
	{
		if(!strcmp(s_f_symbolTable[i].name,sname))
			break;
	}
	return i;
}

int FormulaNumber(char *fname)
{
int i;

	SkipSpaces(&fname);
	if ((!fname[0]) || (fname[0] == '\n')) /* name is empty */
	{
		return s_f_curFNum;
	}
	for(i=0;i<s_f_fNum;i++)
	{
		if(!strcmp(s_f_fName[i], fname))
			break;
	}
	s_f_curFNum = i;
	return i;
}

int DecomposeFrac(s_f_FB **fracFBP, s_f_FB **numFBP, s_f_FB **denomFBP)
{
	if (fracFBP[0]->type != BlockType)
		goto error;
	if (fracFBP[0]->next->type != BlockType)
		goto error;
	if (fracFBP[0]->next->operand != DivOp)
		goto error;

	*numFBP = fracFBP[0]->block;
	*denomFBP = fracFBP[0]->next->block;
	return s_NoErr;
	
error:
	s_printf("Error in DecomposeFrac:\n Not Fractional Formula\n");
	return -1;
}

#if 0 /* obsolete */
int GetFormulaAndSymbolNum(char *procName, char *formulaString, char *symbolString,
							int *formulaNumP, int *symbolNumP)
{
int error = 0;

	if ((*formulaNumP = FormulaNumber(formulaString)) >= s_f_fNum )
	{
		s_printf("Error in %s: %s is not defined\n", procName,formulaString);
		error = 1;
	}
	if ((*symbolNumP = SymbolNumber(symbolString)) >= s_f_sNum )
	{
		s_printf("Error in %s: %s is not defined\n", procName, symbolString);
		error = 1;
	}
	
	return error;
}
#endif

s_f_FB *ComposeTwoFB(s_f_FB *firstFB, s_f_FB *secondFB, int operand)
{
s_f_FB *retFB;

	retFB = NewFormulaBlock();
	retFB->type = BlockType;
	retFB->block = firstFB;
	
	MakeNextBlock(retFB);
	retFB->next->type = BlockType;
	retFB->next->block = secondFB;
	retFB->next->operand = operand;
	
	switch (operand)
	{
	case AddOp:
	case SubOp:
		retFB->operand = AddOp;
		break;
	case MulOp:
	case DivOp:
		retFB->operand = MulOp;
		break;
	}
	
	return retFB;
}

s_f_FB *ComposeCopyTwoFB(s_f_FB *firstFB, s_f_FB *secondFB, int operand)
{
s_f_FB *firstCopyFB, *secondCopyFB;

	firstCopyFB = CopyFB(firstFB);
	secondCopyFB = CopyFB(secondFB);
		
	return ComposeTwoFB(firstCopyFB, secondCopyFB, operand);
}


void InsertFB(s_f_FB *prevFB, s_f_FB **inFBP, s_f_FB *nextFB, s_f_FB **topFBP)
{
s_f_FB *inFB;

	inFB = *inFBP;
	
	*inFBP = inFB->next;
	
	ConcatFB(prevFB, inFB);
	ConcatFB(inFB, nextFB);

	if (nextFB == *topFBP)
	{
		*topFBP = inFB;		
	}
		
}

void ConcatFB(s_f_FB *fb1, s_f_FB *fb2)
{
	if (fb1)
		fb1->next = fb2;
	if (fb2)
		fb2->prev = fb1;
}

int InSet(int n, int sum, int *array)
{
int i;

	for (i = 0; i < sum ; i++)
	{
		if (n == array[i])
			return 1;
	}
	return 0;
}

int OrderCompareBases(int *power1, int *power2)
{
int i = 0;

	for(i = 0; i < s_f_sNum; i++)
	{
		if (power1[s_f_compareOrder[i]] > power2[s_f_compareOrder[i]])
			return 1;
		else if (power1[s_f_compareOrder[i]] < power2[s_f_compareOrder[i]])
			return -1;
	}
	return 0;
}



#endif /*pragma once*/
