/*
 *  Plot.h 
 *  Version 2.0.2 / 011212
 *  by Satoshi Takahashi, Yuriko Hayashi
 */

#ifndef PLOT_H /*pragma once*/
#define PLOT_H

#ifndef SCLLocal
#include <s_Config.h>
#else
#include "s_Config.h"
#endif

typedef void (*voidvoidFunc)(void);

#define SetPlot1O(vn, vx) SetPlot4O(vn, vx, 0, 0, 0)
#define SetPlot2O(vn, vx, vy) SetPlot4O(vn, vx, vy, 0, 0)
#define SetPlot3O(vn, vx, vy, vz) SetPlot4O(vn, vx, vy, vz, 0)
void	SetPlot4O(int vn, double vx, double vy, double vz, double vu);
void	SetPlot6O(int vn, double vx, double vy, double vz, double vu, double vr, double vs);
#define SetPlot2D2O(vx, vy) SetPlot4O(0, vx, vy, 0, 0)

#define SetPlot1W(vn, vx) SetPlot4W(vn, vx, 0, 0, 0)
#define SetPlot2W(vn, vx, vy) SetPlot4W(vn, vx, vy, 0, 0)
#define SetPlot3W(vn, vx, vy, vz) SetPlot4W(vn, vx, vy, vz, 0)
void	SetPlot4W(double vn, double vx, double vy, double vz, double vu);
void	SetPlot6W(double vn, double vx, double vy, double vz, double vu, double vr, double vs);
#define SetPlot2D2W(vx, vy) SetPlot4W(0, vx, vy, 0, 0)
#define SetPlotAW(w) SetPlot4W(w, w, w, w, w)
#define SetPlotHVW(vn, w) SetPlot4W(vn, w, w, w, w)

#define SetPlot1B(vn, vx) SetPlot4B(vn, vx, 0, 0, 0)
#define SetPlot2B(vn, vx, vy) SetPlot4B(vn, vx, vy, 0, 0)
#define SetPlot3B(vn, vx, vy, vz) SetPlot4B(vn, vx, vy, vz, 0)
void	SetPlot4B(int vn, int vx, int vy, int vz, int vu);
void	SetPlot6B(int vn, int vx, int vy, int vz, int vu, int vr, int vs);
#define SetPlot2D2B(vx, vy) SetPlot4B(0, vx, vy, 0, 0)

#define SetPlot1I(x) SetPlot4I(x, 0, 0, 0)
#define SetPlot2I(x, y) SetPlot4I(x, y, 0, 0)
#define SetPlot3I(x, y, z) SetPlot4I(x, y, z, 0)
void	SetPlot4I(double x, double y, double z, double u);
void	SetPlot6I(double x, double y, double z, double u, double r, double s);
void	SetPlot2D2I(double x, double y);

void	Plot1(double x);
void	Plot2(double x, double y);
void	Plot3(double x, double y, double z);
void	Plot4(double x, double y, double z, double u);
void	Plot6(double x, double y, double z, double u, double r, double s);
void	Plot2D2(double x, double y);
void	Plot2D2P(double x, double y);
void	PlotNewPage(void);

void	PlotFrame1D(double n1, double n2, double x1, double x2);
void	PlotFrame2D(double x1, double x2, double y1, double y2);
void	PlotFrame2DSeparate(double x1, double x2, double y1, double y2,
							double x3, double x4, double y3, double y4);


void	SetCoordinateDigit(int n);

void	SetFrame2DXMinDigit(int digit);
void	SetFrame2DXMaxDigit(int digit);
void	SetFrame2DYMinDigit(int digit);
void	SetFrame2DYMaxDigit(int digit);

void	SetColorPlotOn(void);
void	SetColorPlotOff(void);
void	ResetPen(void);

void	PlotPaintCircle(double w);
void	PlotCircle(double w);
void	PlotPaintRect(double w);
void	PlotRect(double w);
void	PlotCross(double w);
void	PlotPaintTriangle(double w);
void	PlotTriangle(double w);

#ifndef PlotCirclePaint
#define PlotCirclePaint PlotPaintCircle
#endif
#ifndef PlotRectPaint
#define PlotRectPaint PlotPaintRect
#endif


void	SetPlotNewPageFunc(voidvoidFunc func);

#if !S_UNIX
void	EnlargePlot2D(void);
#else
#define	EnlargePlot2D(a)
#endif
void	ShrinkPlot2D(double cw);
void	ShrinkPlot2DHalf(void);

#define MagnifyPlot2D EnlargePlot2D

void	Plot2DMoveTo(double x,double y);
void	Plot2DLineTo(double x,double y);
int		Plot2DTransX(double x);
int		Plot2DTransY(double y);
double Plot2DReverseX(short px);
double Plot2DReverseY(short py);

void	BeginPlotPrinting(void);
void	EndPlotPrinting(void);
void	BeginPlotBitMapPrinting(void);
void	EndPlotBitMapPrinting(void);
void	BeginPlotSaving(void);
void	EndPlotSaving(void);
void	BeginPlotBitMapSaving(void);
void	EndPlotBitMapSaving(void);

extern double s_p_scale;
extern long s_n,s_on,s_bn,s_bx,s_by,s_bz,s_bu,s_pflag, s_sflag,s_color_flag;
extern double s_ox,s_oy,s_oz,s_ou,s_wn,s_wx,s_wy,s_wz,s_wu,s_xx,s_yy,s_zz,s_uu;

#define SetPlotPenSize1(pen1) SetPlotPenSize4(pen1,s_p_pen2,s_p_pen3,s_p_pen4)
#define SetPlotPenSize2(pen1,pen2) SetPlotPenSize4(pen1,pen2,s_p_pen3,s_p_pen4)
#define SetPlotPenSize3(pen1,pen2,pen3) SetPlotPenSize4(pen1,pen2,pen3,s_p_pen4)
void	SetPlotPenSize4(int pen1,int pen2,int pen3,int pen4);
void	SetPlotPenSize6(int pen1,int pen2,int pen3,int pen4,int pen5,int pen6);

int		PlotPointInRegion(double x, double y);

void	SetPlotNewPageStopOn();
void	SetPlotNewPageStopOff();


#ifndef SCLLocal
#include <VarHeader.h>
#include <Plot-v.c>
#else
#include "VarHeader.h"
#include "Plot-v.c"
#endif


#endif /*pragma once*/
