/*
 *  SCLDrawingFile.c
 *  Version 1.5.1 / 980112
 *  by Satoshi Takahashi, Yumiko Takahashi & Hidekazu Koike
 *  Include Save.h and Plot.h
 */

#ifndef SCL_DRAWING_FILE_C /*pragma once*/
#define SCL_DRAWING_FILE_C
#include <s_Config.h>

#if !UNIX

#include <stdio.h>
#include <string.h>

#include <SCLDrawingFile.h>
#include <StartSCL.h>
#include <DialogFOpen.h>
#include <s_RedrawFunc.h>
#include <Save.h>
#include <Plot.h>
#include <s_define.c>

s_sdfType s_sdf =
{
	0,		/* fp */
	1023,	/* maxLineNum */
	"",		/* oneLine[1024] */
	"main",	/* stopString[64] */
	1,		/* skipLines */
	"}",	/* lastString[64] */
	{0,0,0,0},/* Rect r */
};

typedef struct
{
	char* name;
	voidStringFunc *func;
} sdfCommandTableStruct;


sdfCommandTableStruct s_sdfCommandTable[] = 
{
	{"Start",				DummyVoidString},
	{"StartColor",			DummyVoidString},
	{"MoveTo",				DoMoveTo},
	{"LineTo",				DoLineTo},
	{"Stop",				DummyVoidString},
	{"PenSize",				DoPenSize},
	{"EraseDrawing",		DoEraseDrawing},
	{"SetRect",				DoSetRect},
	{"FrameRect",			DoFrameRect},
	{"PaintRect",			DoPaintRect},
	{"EraseRect",			DoEraseRect},
	{"InvertRect",			DoInvertRect},
	{"FrameOval",			DoFrameOval},
	{"PaintOval",			DoPaintOval},
	{"EraseOval",			DoEraseOval},
	{"InvertOval",			DoInvertOval},
	{"DrawText",			DoDrawText},
	{"BeginBitMapSaving",	DoBeginBitMapSaving},
	{"EndBitMapSaving",		DoEndBitMapSaving},
	{"BeginPlotSaving",		DoBeginPlotSaving},
	{"EndPlotSaving",		DoEndPlotSaving},
	{"BeginSaving",			DoBeginSaving},
	{"EndSaving",			DoEndSaving},

	{0,				DummyVoidString}, /* this line must be at last */
};


/******************** DrawSCLFile ********************/

void DrawSCLFile(void)
{
	s_sdf.fp =  DialogFOpen("SCLDrawingData.c", "r");
	
	if (!s_sdf.fp)
		return;
	
	SkipDF();
	
	while (fgets(s_sdf.oneLine, s_sdf.maxLineNum, s_sdf.fp))
	{
		if (!strncmp(s_sdf.oneLine, s_sdf.lastString, strlen(s_sdf.lastString)))
			break;
		
		DoDrawCommand(s_sdf.oneLine);
	}
	fclose(s_sdf.fp);
}


void	SkipDF(void)
{
int i;

	while (fgets(s_sdf.oneLine, s_sdf.maxLineNum, s_sdf.fp))
	{
		if (!strncmp(s_sdf.oneLine, s_sdf.stopString, strlen(s_sdf.stopString)))
			break;
	}
	
	for (i = 0; i < s_sdf.skipLines ;i++)
		fgets(s_sdf.oneLine, s_sdf.maxLineNum, s_sdf.fp);
}


void DoDrawCommand(char *oneLine)
{
char commandName[64];
voidStringFunc *commandFunc;

	sscanf(oneLine, "%[^(]", commandName);
	commandFunc = SDFSelectCommand(commandName);
	(*commandFunc)(oneLine);
}


voidStringFunc *SDFSelectCommand(char *commandString)
{
int i, length, maxLength = 0, maxNum = 0, maxIndex;
	
	for (i = 0; s_sdfCommandTable[i].name; i++)
	{
		if (!strcmp(s_sdfCommandTable[i].name, commandString))
			return s_sdfCommandTable[i].func;
	}
	
	s_printf("%s is not defined in DrawSCLFile\n", commandString);
	
	return s_sdfCommandTable[i].func; /* return dummy func */
}


void DummyVoidString(char *inputLine)
{
}



/******************** Line ********************/

void DoMoveTo(char *inputLine)
{
int scanNum;
char commandName[64];
short h;
short v;

	scanNum = sscanf(inputLine, "%[^(](%hd,%hd);", commandName, &h, &v);
	if (scanNum == 3)
	{
		s_MoveTo(h, v);
	}
	else
	{
		s_printf("Error in command: %s\n", inputLine);
	}
}


void DoLineTo(char *inputLine)
{
int scanNum;
char commandName[64];
short h;
short v;

	scanNum = sscanf(inputLine, "%[^(](%hd,%hd);", commandName, &h, &v);
	
	if (scanNum == 3)
	{
		LineTo(h, v);
	}
	else
	{
		s_printf("Error in command: %s\n", inputLine);
	}
}

void DoPenSize(char *inputLine)
{
int scanNum;
char commandName[64];
short h;
short v;

	scanNum = sscanf(inputLine, "%[^(](%hd,%hd);", commandName, &h, &v);
	
	if (scanNum == 3)
	{
		PenSize(h, v);
	}
	else
	{
		s_printf("Error in command: %s\n", inputLine);
	}
}


void DoEraseDrawing(char *inputLine)
{
int scanNum;
char commandName[64];

	scanNum = sscanf(inputLine, "%[^(](%hd,%hd);", commandName);
	
	if (scanNum == 1)
	{
		EraseDrawing();
	}
	else
	{
		s_printf("Error in command: %s\n", inputLine);
	}
}


/******************** Rect ********************/

void DoSetRect(char *inputLine)
{
int scanNum;
char commandName[64];
short left, top, right, bottom;

	scanNum = sscanf(inputLine, "%[^(](&r,%hd,%hd,%hd,%hd);", commandName, &left, &top, &right, &bottom);
	
	if (scanNum == 5)
	{
		SetRect(&s_sdf.r, left, top, right, bottom);
	}
	else
	{
		s_printf("Error in command: %s\n", inputLine);
	}
}


void DoFrameRect(char *inputLine)
{
int scanNum;
char commandName[64];

	scanNum = sscanf(inputLine, "%[^(](&r);", commandName);
	
	if (scanNum == 1)
	{
		FrameRect(&s_sdf.r);
	}
	else
	{
		s_printf("Error in command: %s\n", inputLine);
	}
}


void DoPaintRect(char *inputLine)
{
int scanNum;
char commandName[64];

	scanNum = sscanf(inputLine, "%[^(](&r);", commandName);
	
	if (scanNum == 1)
	{
		PaintRect(&s_sdf.r);
	}
	else
	{
		s_printf("Error in command: %s\n", inputLine);
	}
}


void DoEraseRect(char *inputLine)
{
int scanNum;
char commandName[64];

	scanNum = sscanf(inputLine, "%[^(](&r);", commandName);
	
	if (scanNum == 1)
	{
		EraseRect(&s_sdf.r);
	}
	else
	{
		s_printf("Error in command: %s\n", inputLine);
	}
}


void DoInvertRect(char *inputLine)
{
int scanNum;
char commandName[64];

	scanNum = sscanf(inputLine, "%[^(](&r);", commandName);
	
	if (scanNum == 1)
	{
		InvertRect(&s_sdf.r);
	}
	else
	{
		s_printf("Error in command: %s\n", inputLine);
	}
}


/******************** Oval ********************/

void DoFrameOval(char *inputLine)
{
int scanNum;
char commandName[64];

	scanNum = sscanf(inputLine, "%[^(](&r);", commandName);
	
	if (scanNum == 1)
	{
		FrameOval(&s_sdf.r);
	}
	else
	{
		s_printf("Error in command: %s\n", inputLine);
	}
}


void DoPaintOval(char *inputLine)
{
int scanNum;
char commandName[64];

	scanNum = sscanf(inputLine, "%[^(](&r);", commandName);
	
	if (scanNum == 1)
	{
		PaintOval(&s_sdf.r);
	}
	else
	{
		s_printf("Error in command: %s\n", inputLine);
	}
}


void DoEraseOval(char *inputLine)
{
int scanNum;
char commandName[64];

	scanNum = sscanf(inputLine, "%[^(](&r);", commandName);
	
	if (scanNum == 1)
	{
		EraseOval(&s_sdf.r);
	}
	else
	{
		s_printf("Error in command: %s\n", inputLine);
	}
}


void DoInvertOval(char *inputLine)
{
int scanNum;
char commandName[64];

	scanNum = sscanf(inputLine, "%[^(](&r);", commandName);
	
	if (scanNum == 1)
	{
		InvertOval(&s_sdf.r);
	}
	else
	{
		s_printf("Error in command: %s\n", inputLine);
	}
}


/******************** DrawText ********************/

void DoDrawText(char *inputLine)
{
int scanNum;
char commandName[64];
char string[256];
short start,length;

	scanNum = sscanf(inputLine, "%[^(](\"%[^\"]\",%hd,%hd);", commandName,string,&start,&length);
	
	if (scanNum == 4)
	{
		DrawText(string,start,length);
	}
	else
	{
		s_printf("Error in command: %s\n", inputLine);
	}
}


/******************** Save ********************/

void DoBeginBitMapSaving(char *inputLine)
{
int scanNum;
char commandName[64];

	scanNum = sscanf(inputLine, "%[^(](%hd,%hd);", commandName);
	
	if (scanNum == 1)
	{
		BeginBitMapSaving();
	}
	else
	{
		s_printf("Error in command: %s\n", inputLine);
	}
}


void DoEndBitMapSaving(char *inputLine)
{
int scanNum;
char commandName[64];

	scanNum = sscanf(inputLine, "%[^(](%hd,%hd);", commandName);
	
	if (scanNum == 1)
	{
		EndBitMapSaving();
	}
	else
	{
		s_printf("Error in command: %s\n", inputLine);
	}
}


void DoBeginPlotSaving(char *inputLine)
{
int scanNum;
char commandName[64];

	scanNum = sscanf(inputLine, "%[^(](%hd,%hd);", commandName);
	
	if (scanNum == 1)
	{
		BeginPlotSaving();
	}
	else
	{
		s_printf("Error in command: %s\n", inputLine);
	}
}

void DoEndPlotSaving(char *inputLine)
{
int scanNum;
char commandName[64];

	scanNum = sscanf(inputLine, "%[^(](%hd,%hd);", commandName);
	
	if (scanNum == 1)
	{
		EndPlotSaving();
	}
	else
	{
		s_printf("Error in command: %s\n", inputLine);
	}
}


void DoBeginSaving(char *inputLine)
{
int scanNum;
char commandName[64];


	scanNum = sscanf(inputLine, "%[^(]();", commandName);
	
	if (scanNum == 1)
	{
		BeginSaving();
	}
	else
	{
		s_printf("Error in command: %s\n", inputLine);
	}
}


void DoEndSaving(char *inputLine)
{
int scanNum;
char commandName[64];


	scanNum = sscanf(inputLine, "%[^(]();", commandName);
	
	if (scanNum == 1)
	{
		EndSaving();
	}
	else
	{
		s_printf("Error in command: %s\n", inputLine);
	}
}


#endif /*!UNIX*/

#include <s_define.c>

#endif /*pragma once*/