/*
 *  s_File.c
 *  Version 2.0 / 010912
 *  by Satoshi Takahashi
 */


#ifndef S_FILE_C /*pragma once*/
#define S_FILE_C
#ifndef SCLLocal
#include <s_Config.h>
#else
#include "s_Config.h"
#endif

#if S_CARBON
#include <Carbon/Carbon.h>
#endif

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#ifndef SCLLocal

#include <FileCopy.h> /*MoreFiles*/
#include <MoreFilesExtras.h> /*MoreFiles*/
#include <DirectoryCopy.h> /*MoreFiles*/

#include <s_String.h>
#include <DialogFOpen.h> /*s_GetFullPath*/
#include <s_Common.h>
#if DEBUG_SCRIPT
#include <StartSCL.h>
#endif

#include <s_File.h>
#include <VarBody.h>
#include <s_File-v.c>

#else /*ndef SCLLocal*/

#include "FileCopy.h" /*MoreFiles*/
#include "MoreFilesExtras.h" /*MoreFiles*/
#include "DirectoryCopy.h" /*MoreFiles*/

#include "s_String.h"
#include "DialogFOpen.h" /*s_GetFullPath*/
#include "s_Common.h"
#if DEBUG_SCRIPT
#include "StartSCL.h"
#endif

#include "s_File.h"
#include "VarBody.h"
#include "s_File-v.c"

#endif /*else ndef SCLLocal*/


/*************************************************************
	OSType2CString('abcd', string);
	makes string == "abcd".
*************************************************************/

void OSType2CString(OSType type, char *typeStr)
{
OSType *typeP;

	typeP = (OSType *)typeStr;
	*typeP = type;
	typeStr[4] = 0;
}


void CString2OSType(char *typeStr, OSType *typeP)
{
OSType *tmpTypeP;

	tmpTypeP = (OSType *)typeStr;
	*typeP = *tmpTypeP;
}


/*************************************************************
	IsFolder("abc:def");
	returns 1 if "abc:def" is folder,
	and 0 if it is not a folder.
*************************************************************/

int IsFolder(char *fullPathName)
{
Str255 pPathName, retFileName;
short vRefNum;
long dirID;
Boolean isDirectory;

	C2PString(fullPathName, pPathName);
	GetObjectLocation(0, 0, pPathName,
						&vRefNum, &dirID, retFileName,
							&isDirectory);
	return isDirectory;
}


#if !S_CARBON
int SFpIsFolder(FSSpec *fssP)
{
long dirID;
Boolean isDirectory;

	//DirIDFromFSSpec(fssP, &dirID, &isDirectory);
	FSpGetDirectoryID(fssP, &dirID, &isDirectory);
	return isDirectory;
}
#endif /*!S_CARBON*/
/*************************************************************
	SFGetBaseAndDirName("abc:de:fghi", baseName, dirName)
	makes baseName == "fghi", dirName == "abc:de:"
*************************************************************/

void SFGetBaseAndDirName(char* pathString,
		char *baseName,char *dirName)
{
int i;
int pos = 0;
#if S_CARBON
char separator = '/';
#else
char separator = ':';
#endif

	for (i = strlen(pathString)-1; i >=0 ; i--)
	{
		if (pathString[i] == separator)
		{
			pos = i+1; /*next char of ':'*/
			break;
		}
	}
	
	if (baseName)
	{
		strcpy(baseName, pathString + pos);
		strncpy(dirName, pathString, pos);
	}
	if (dirName)
		dirName[pos] = 0;
}


void SFGetBodyAndExtension(char* fileName,
		char *body,char *extension)
{
int i;

	for (i = strlen(fileName)-1; i >=0 ; i--)
	{
		if (fileName[i] == '.')
			break;
	}
	
	if (i > 0)
	{
		if (body)
		{
			strncpy(body, fileName, i);
			body[i] = 0;
		}
		if (extension)
			strcpy(extension, fileName + i + 1);
	}
	else
	{
		if (body)
			strcpy(body, fileName);
		if (extension)
			extension[0] = 0;
	}
}


/************************************************************
	SFFileCopy("abc:de", "ijk:lm");
	or SFFileCopy("abc:de", "ijk:lm:");
	destFile may be either file name or folder name
*************************************************************/

OSErr SFFileCopy(char *sourceFile, char *destFile)
{
	return SFFileCopyOrMove(sourceFile, destFile, 1); /* 1 is copy */
}


/************************************************************
	SFFileMove("abc:de", "ijk:lm");
	or SFFileMove("abc:de", "ijk:lm:");
	destFile may be either file name or folder name
*************************************************************/

OSErr SFFileMove(char *sourceFile, char *destFile)
{
	return SFFileCopyOrMove(sourceFile, destFile, 0); /* 0 is move */
}


/************************************************************
	OSErr SFFileCopyOrMove(char *sourceFile, char *destFile, int copyFlag)
	sourceFile: full path of source file
	destFile: full path of destinate file or directory
	copyFlag: 0 is move, 1 is copy
*************************************************************/

OSErr SFFileCopyOrMove(char *sourceFile, char *destFile, int copyFlag)
{
char *buf = 0;
long bufSize = 32*1024L;
char destFullName[256], sourceName[64], sourcePath[256],
		destName[64], destPath[256], tmpsourceFile[256];
Str255 PsourceFile, PdestFile, PdestFullName,
		PdestName, PdestPath;
OSErr err;
FSSpec sourceFSS;

	if (err = SFCheckExist(sourceFile, &sourceFSS))
		return err;

	if (copyFlag)
	{
		if (! (buf = malloc(bufSize)))
		{
			return -1;
		}
	}
		
	if (IsFolder(destFile)) /*destFile is folder name*/
	{
		SFGetBaseAndDirName(sourceFile, sourceName, sourcePath);
		if (destFile[strlen(destFile)-1] != ':')
			sprintf(destFullName,"%s:%s", destFile, sourceName);
		else
			sprintf(destFullName,"%s%s", destFile, sourceName);
		C2PString(destFullName, PdestFullName);
		err = HDelete(0, 0, PdestFullName);

		C2PString(sourceFile, PsourceFile);
		C2PString(destFile, PdestFile);
		if (copyFlag)
			err = FileCopy(0, 0, PsourceFile, 0, 0, PdestFile, 0,
				buf, bufSize, true);
		else
			err = CatMove(0, 0, PsourceFile, 0, PdestFile);
	}
	else /*destFile is file name*/
	{
		C2PString(destFile, PdestFile);
		err = HDelete(0, 0, PdestFile);

		SFGetBaseAndDirName(destFile, destName, destPath);
		C2PString(sourceFile, PsourceFile);
		C2PString(destName, PdestName);
		C2PString(destPath, PdestPath);
		
		if (copyFlag)
			err = FileCopy(0, 0, PsourceFile, 0, 0, PdestPath, PdestName,
				buf, bufSize, true);
		else
		{
			SFGetBaseAndDirName(sourceFile, sourceName, sourcePath);
			if (strcmp(sourceName, destName))
			{
				err = HRename(0, 0, PsourceFile, PdestName);
				sprintf(tmpsourceFile, "%s%s", sourcePath, destName);
					/*sourcePath contains ':' at its tail*/
				C2PString(tmpsourceFile, PsourceFile);
			}
			if (strcmp(sourcePath, destPath))
			{
				err = CatMove(0, 0, PsourceFile, 0, PdestPath);
#if DEBUG_SCRIPT
s_printf("tmpsourceFile = %s\n", tmpsourceFile);
s_printf("CatMove-error = %hd\n", err);
#endif
			}
		}
	}
	
	if (buf)
	{
		free(buf);
		buf = 0;
	}
	return err;
}


/************************************************************
	modifiedTime = GetModDate("abc:de");
	returns modified time in sec from 1904.01.01 00:00
*************************************************************/

unsigned long GetModDate(char *fileFullName)
{
Str255 name;
CInfoPBRec pb;
OSErr err;

	C2PString(fileFullName, name);
	pb.hFileInfo.ioNamePtr = name;
	pb.hFileInfo.ioVRefNum = 0;
	pb.hFileInfo.ioDirID = 0;
	pb.hFileInfo.ioFDirIndex = 0;	/* use ioNamePtr and ioDirID */
	err = PBGetCatInfoSync(&pb);
	
	return pb.hFileInfo.ioFlMdDat;
}


unsigned long SFpGetModDate(FSSpec *fssP)
{
CInfoPBRec pb;
OSErr err;

	pb.hFileInfo.ioNamePtr = fssP->name;
	pb.hFileInfo.ioVRefNum = fssP->vRefNum;
	pb.hFileInfo.ioDirID = fssP->parID;
	pb.hFileInfo.ioFDirIndex = 0;
	err = PBGetCatInfoSync(&pb);
	
	return pb.hFileInfo.ioFlMdDat;
}


/************************************************************
	SFUpdate("abc:source", "def:dest");
	copy abc:source to def:dest only if abc:dest
	is newly modified.
*************************************************************/

#define SFBufSize (128*1024L)

#if !S_CARBON || 1
void SFUpdate(char *sourceFile, char *destFile)
{
FSSpec sourceFSS, destFSS;

char *buf = 0;

	if (SFCheckExist(sourceFile, &sourceFSS))
		return;
	if (SFCheckExist(destFile, &destFSS))
		return;

	if (! (buf = malloc(SFBufSize)))
		return;
#if DirUpdate
	if (IsFolder(sourceFile))
	{
		if (!IsFolder(destFile))
		{
			FSpDelete(&destFSS);
			DirectoryCopy(sourceFSS.vRefNum, 
				sourceFSS.parID, sourceFSS.name, 
				destFSS.vRefNum, destFSS.parID, 0, 0,
				buf, SFBufSize, true, 0);
		}
		else
			SFDirUpdate(&sourceFSS, &destFSS, buf);
	}
	else
#endif
	{
		if (IsFolder(destFile))
		{
			DeleteDirectory(destFSS.vRefNum, destFSS.parID,
						destFSS.name);
			FSpFileCopy(&sourceFSS, &destFSS, 0, buf, SFBufSize, true);
		}
		else
			SFFileUpdate(&sourceFSS, &destFSS, buf);
	}

	if (buf)
	{
		free(buf);
		buf = 0;
	}
}
#endif

void SFFileUpdate(FSSpec *sourceFSSP, FSSpec *destFSSP, char *buf)
{
unsigned long sourceTime, destTime;

	sourceTime = SFpGetModDate(sourceFSSP);
	destTime = SFpGetModDate(destFSSP);
#if DEBUG_SCRIPT
s_printf("%#s, %lu, %lu\n", sourceFSSP->name, sourceTime, destTime);
#endif
	if (sourceTime > destTime)
	{
		FSpDelete(destFSSP);
		FileCopy(sourceFSSP->vRefNum, sourceFSSP->parID, sourceFSSP->name,
					destFSSP->vRefNum, destFSSP->parID, 0, 0,
					buf, SFBufSize, true);
	}
}


#define FileItemMax 256
#define DirItemMax 16

#if !S_CARBON || 1
#if DirUpdate

void SFDirUpdate(FSSpec *sourceFSSP, FSSpec *destFSSP, char *buf)
{
	SFDirUpdateIn(sourceFSSP, destFSSP, buf, 0);
	SFDirUpdateIn(sourceFSSP, destFSSP, buf, 1);
}

void SFDirUpdateIn(FSSpec *sourceFSSP, FSSpec *destFSSP, char *buf, int dirFlag)
{
FSSpec *itemList;
FSSpec fss;
short itemNum;
short itemIndex = 1;
int count;
short itemMax;

long dirID;
Boolean isDirectory;
OSErr error;

	if (!dirFlag)
		itemMax = FileItemMax;
	else
		itemMax = DirItemMax;
	
	itemNum = itemMax;
	
	itemList = malloc(itemMax*sizeof(FSSpec));
	if (!itemList)
		return;

	//DirIDFromFSSpec(sourceFSSP, &dirID, &isDirectory);
	FSpGetDirectoryID(sourceFSSP, &dirID, &isDirectory);

	do
	{
#if DEBUG_SCRIPT
s_printf("itemMax=%hd\n", itemMax);
s_printf("destName=%#s\n",destFSSP->name);
#endif
		error = GetDirItems(destFSSP->vRefNum, destFSSP->parID, destFSSP->name,
				!dirFlag /*get files*/, dirFlag /*no directories*/, itemList,
				itemMax, &itemNum, &itemIndex);
		/* if GetDirItems returns noErr, there are more items!!! */
#if DEBUG_SCRIPT
s_printf("itemNum=%ld\n", itemNum);
s_printf("name0=%#s\n", itemList[0].name);
s_printf("name1=%#s\n", itemList[1].name);
#endif
		for (count = 0; count < itemNum; count++)
		{
#if DEBUG_SCRIPT
s_printf("%d\n", count);
#endif
			if (FSMakeFSSpec(sourceFSSP->vRefNum, dirID,
					itemList[count].name, &fss))
			{
#if DEBUG_SCRIPT
s_printf("%d delete %#s\n", count, itemList[count].name);
#endif
				if (!dirFlag)
					FSpDelete(&itemList[count]);
				else
					DeleteDirectory(itemList[count].vRefNum, itemList[count].parID,
								itemList[count].name);
			}
		}
#if DEBUG_SCRIPT
s_printf("itemMax=%ld\n", itemMax);
#endif
	} while (error == noErr);
#if DEBUG_SCRIPT
s_printf("after\n");
#endif

	itemIndex = 1;
	//DirIDFromFSSpec(destFSSP, &dirID, &isDirectory);
	FSpGetDirectoryID(destFSSP, &dirID, &isDirectory);
	do
	{
		error = GetDirItems(sourceFSSP->vRefNum, sourceFSSP->parID, sourceFSSP->name,
				!dirFlag /*get files*/, dirFlag /*no directories*/, itemList,
				itemMax, &itemNum, &itemIndex);
		for (count = 0; count < itemNum; count++)
		{
			if (FSMakeFSSpec(destFSSP->vRefNum, dirID,
					itemList[count].name, &fss))
			{
#if DEBUG_SCRIPT
s_printf("copy %#s\n", itemList[count].name);
#endif
				if (!dirFlag)
					FSpFileCopy(&itemList[count], destFSSP, 0,
									buf, SFBufSize, true);
				else
				{
					FSpDirectoryCopy(&itemList[count], destFSSP, 0,
										buf, SFBufSize, true, 0);
				}
			}
			else
			{
				if (!dirFlag)
				{
					SFFileUpdate(&itemList[count], &fss, buf);
				}
				else
					SFDirUpdate(&itemList[count], &fss, buf);
			}
		}
	} while (error == noErr);

	free(itemList);
}
#endif /*DirUpdate?*/
#endif /*!S_CARBON*/


/************************************************************
	SFSetCreator("abc:de", 'uvwx');
	sets the creator of the file abc:de to 'uvwx'
*************************************************************/

OSErr SFSetCreator(FSSpec *fileSpecP, OSType destType)
{
FInfo finderInfo;
OSErr error;

	if (!(error = FSpGetFInfo(fileSpecP, &finderInfo)))
	{
		finderInfo.fdCreator = destType;
		error = FSpSetFInfo(fileSpecP, &finderInfo);
	}
	return error;
}


/************************************************************
	SFSetType("abc:de", 'uvwx');
	sets the type of the file abc:de to 'uvwx'
*************************************************************/

OSErr SFSetType(FSSpec *fileSpecP, OSType destType)
{
FInfo finderInfo;
OSErr error;

	if (!(error = FSpGetFInfo(fileSpecP, &finderInfo)))
	{
		finderInfo.fdType = destType;
		error = FSpSetFInfo(fileSpecP, &finderInfo);
	}
	return error;
}

#if 0
void SetType(FSSpec *fileSpecP, OSType destType)
{	
	SFSetType(fileSpecP, destType);
}
#endif


/************************************************************
	creator = SFGetCreator("abc:de");
	get the creator of the file abc:de
*************************************************************/

OSType SFGetCreator(FSSpec *fileSpecP)
{
FInfo finderInfo;
OSErr err;

	if (!(err = FSpGetFInfo(fileSpecP, &finderInfo)))
		return finderInfo.fdCreator;
	else
		return 0;
}


/************************************************************
	type = SFGetCreator("abc:de");
	get the type of the file abc:de
*************************************************************/

OSType SFGetType(FSSpec *fileSpecP)
{
FInfo finderInfo;
OSErr err;

	if (!(err = FSpGetFInfo(fileSpecP, &finderInfo)))
		return finderInfo.fdType;
	else
		return 0;
}


/************************************************************
	SFFullPath(&fss, fullPath);
	sets the full path of file fss into fullPath
*************************************************************/

void SFFullPath(FSSpec *fssP, char *fullPath, long pathMax)
{
#if S_CARBON
FSRef fsr;
	FSpMakeFSRef(fssP, &fsr);
	FSRefMakePath(&fsr, fullPath, pathMax);
#else /*S_CARBON*/
char fileName[64];

	P2CString(fssP->name, fileName);
	sprintf(fullPath, "%s%s", s_GetFullPath(fssP->parID,fssP->vRefNum),
					fileName);
#endif /*else S_CARBON*/
}

/************************************************************
	void SFPath2Spec(char *fullPath, FSSpec *fssP)
	convert from fullpath to FSSpec
*************************************************************/

OSErr SFPath2Spec(char *fullPath, FSSpec *fssP)
{
#if S_CARBON
FSRef fsr;
Boolean isDirectory;
OSErr error;

	error = FSPathMakeRef(fullPath, &fsr, &isDirectory);
	if (!error)
		error = FSGetCatalogInfo(&fsr, kFSCatInfoNone, 0, 0, fssP, 0);
	return error;
#else /*S_CARBON*/
Str255 pFileName;

	C2PString(fullPath, pFileName);
	return FSMakeFSSpec(0, 0, pFileName, fssP);
#endif /*else S_CARBON*/
}


/************************************************************
	OSErr SFCheckExist(char *filePath)
	check if the file is exist
*************************************************************/

OSErr SFCheckExist(char *filePath, FSSpec *fssP)
{
OSErr error;
Str255 PFile, errorString;

	C2PString(filePath, PFile);
	error = FSMakeFSSpec(0, 0, PFile, fssP);
	if (error==nsvErr || error==fnfErr)
	{
		ParamText(PFile, "\pnot found!", 0, 0);
		CautionAlert(129, 0);
		return -1;
	}
	else if (error)
	{
		NumToString(error, errorString);
		ParamText("\pUnknown error for", PFile,
					"\perror code", errorString);
		CautionAlert(129, 0);
		return -1;
	}
	return 0;
}



int SFExist(char *filePath)
{
FSSpec fss;

	return !SFPath2Spec(filePath, &fss);
}


void SFRemoveDirectory(char *filePath)
{
FSSpec fss;

	if (SFCheckExist(filePath, &fss))
		return;
	DeleteDirectory(fss.vRefNum, fss.parID, fss.name);
}


#if !S_CARBON
void SFAllFileDo(FSSpec *targetFolderP, OSErr (*doFunc)(FSSpec *targetFile))
{
	if (SFpIsFolder(targetFolderP))
	{
		s_f.allDoFunc = doFunc;
		SFAllFileRecurse(targetFolderP);
	}
	else
		doFunc(targetFolderP);
}


void SFAllFileRecurse(FSSpec *sourceFSSP)
{
	SFAllFileRecurseIn(sourceFSSP, 0);
	SFAllFileRecurseIn(sourceFSSP, 1);
}

void SFAllFileRecurseIn(FSSpec *sourceFSSP, int dirFlag)
{
FSSpec *itemList;
FSSpec fss;
short itemNum;
short itemIndex = 1;
int count;
int destLen;
long itemMax;
long dirID;
Boolean isDirectory;
OSErr error;

	if (!dirFlag)
		itemMax = FileItemMax;
	else
		itemMax = DirItemMax;
	
	itemNum = itemMax;
	
	itemList = malloc(itemMax*sizeof(FSSpec));
	if (!itemList)
		return;

	DirIDFromFSSpec(sourceFSSP, &dirID, &isDirectory);


	itemIndex = 1;
	do
	{
		error = GetDirItems(sourceFSSP->vRefNum, sourceFSSP->parID, sourceFSSP->name,
				!dirFlag /*get files*/, dirFlag /*no directories*/, itemList,
				itemMax, &itemNum, &itemIndex);
		for (count = 0; count < itemNum; count++)
		{
				if (!dirFlag)
				{
					error = s_f.allDoFunc(&itemList[count]);
				}
				else
					SFAllFileRecurse(&itemList[count]);
		}
	} while (error == noErr);

	free(itemList);
}
#endif

#endif /*pragma once*/