/*
 *  Print.c
 *  Version 2.0.3 / 020727
 *  by Satoshi Takahashi & Akiya Iwasaki
 */


#ifndef S_PRINT_C /*pragma once*/
#define S_PRINT_C
#include <s_Config.h>
#if S_CARBON
#include <Carbon/Carbon.h>
#endif
#include <s_Carbon.h>
#include <s_CarbonMacro.h>

#define S_SATOSHI2

#include <string.h>
#include <stdlib.h>

#if !S_UNIX && !S_CARBON
#include <QDOffScreen.h>
#endif

#include <StartSCL.h>
#include <Save.h>
#include <s_Common.h>
#include <s_RedrawFunc.h>
#include <s_QDGlobal.h>
#include <s_String.h>

#include <s_undef.c>

#if S_UNIX /* as s_UNIX.h is not in parts-main */
#include <s_UNIX.h>
#endif
#include <s_Print.h>
/*#include <s_undef.c>*/

#include <VarBody.h>
#include <s_Print-v.c>


/***** Definition of Variables and Functions *****/

/* This part should be included only in one file */
/* To avoid including, write "#define S_SUB"     */
/* before "#include <satoshi.2h>"                */

void BeginPrinting(void)
{
#if !S_UNIX
GrafPtr currPort;
GDHandle currDev;

	if (s_save.openPictureFlag)
		PrintError("Error: Consecutive call of BeginSaving or BeginPrinting\n");

	s_GetGWorld((GrafPtr*)&currPort, &currDev);
	s_SetGWorld(GetWindowPort(s_drawingWindow), nil);
	if (s_print.pict)
    {
        KillPicture(s_print.pict);
        s_print.pict = 0;
    }
	ClipRect(SCWindowBounds(s_drawingWindow));
	s_print.pict = OpenPicture(SCWindowBounds(s_drawingWindow));
	s_save.openPictureFlag = 1;
	s_ShowPen();
	s_SetGWorld(currPort, currDev);
	CheckPause();
#else/*!S_UNIX*/
	if (s_UNIX.drawingFlag)
		fprintf(s_UNIX.drawingFile, "BeginPrinting();\n");
#endif
}

int EndPrinting(void) 
{
#if !S_UNIX
GWorldPtr currPort;
GDHandle currDev;
int doPrint;

	s_GetGWorld((GrafPtr*)&currPort, &currDev);
	s_SetGWorld(GetWindowPort(s_drawingWindow), nil);
	s_HidePen();
	ClosePicture();
	s_save.openPictureFlag = 0;
	doPrint = PrintPict(s_print.pict, s_noBlind);
	s_SetGWorld((GrafPtr)currPort, currDev);
	CheckPause();
	return doPrint;
#else/*!S_UNIX*/
char command[50];
char userReply[256];

	if (s_UNIX.drawingFlag)
	{
#if 0 /* No merit for asking in UNIX */
		if (s_Config.foreground)
		{
			printf("Do you print graphic? (y/n): ");
			fgets(userReply, 256, stdin);

			if ( CancelName(userReply) )
				return 0;
		}
#endif

		fprintf(s_UNIX.drawingFile, "EndPrinting();\n");
		if (s_Config.XFlag)
		{
			sprintf(command, "xwd -id %d|xpr|lp", thePort->win);
			system(command);
		}
	}
	
	return 1;
#endif
}

void EndPrintingBlind(void) 
{
#if !S_UNIX
GrafPtr currPort;
GDHandle currDev;

	s_GetGWorld((GrafPtr*)&currPort, &currDev);
	s_SetGWorld(GetWindowPort(s_drawingWindow), nil);
	s_HidePen();
	ClosePicture();
	s_save.openPictureFlag = 0;
	PrintPict(s_print.pict, s_blind);
	s_SetGWorld(currPort, currDev);
	CheckPause();
#else/*!S_UNIX*/
char command[50];

	if (s_UNIX.drawingFlag)
	{
		fprintf(s_UNIX.drawingFile, "EndPrintingBlind();\n");
		if (s_Config.XFlag)
		{
			sprintf(command, "xwd -id %d|xpr|lp", thePort->win);
			system(command);
		}
	}
#endif
}

#if !S_UNIX /******************** !!!! UNIX ********************/
void EndPrintingSamePageSetUp(void) 
{
GrafPtr currPort;
GDHandle currDev;

	s_GetGWorld((GrafPtr*)&currPort, &currDev);
	s_SetGWorld(GetWindowPort(s_drawingWindow), nil);
	s_HidePen();
	ClosePicture();
	PrintPict(s_print.pict, s_samePageSetUp);
	s_SetGWorld(currPort, currDev);
	CheckPause();
}

#if S_CARBON

/*extern OSStatus PMBegin(void);*/

int	PrintPict(PicHandle pict,int mode)
{
Boolean doPrint;
GWorldPtr currPort;
GDHandle currDev;
OSErr error;

    if (PMCreateSession(&s_print.session))
        return 0;
	s_print.prResFile = CurResFile();

    error = PageSetUp(mode, s_print.session, &s_print.format);

	s_GetGWorld((GrafPtr*)&currPort, &currDev);
	s_SetGWorld(GetWindowPort(s_displayWindow), s_display);

    if (!(error = PMCreatePrintSettings(&s_print.setting)))
        error = PMSessionDefaultPrintSettings(s_print.session, s_print.setting);

	if (!error && mode==s_noBlind)
    {
		error = PMSessionPrintDialog(s_print.session, s_print.setting, s_print.format, &doPrint);
	}
    else
		doPrint = !error;
	s_SetGWorld((GrafPtr)currPort, currDev);
	if (!error && doPrint)
        DoPrintJob(&PictProc, pict, 0);
#if 0 /*format may be used later*/
	if (s_print.format)
        PMRelease(s_print.format);
#endif
    if (s_print.setting)
    {
        PMRelease(&s_print.setting);
        s_print.setting = 0;
    }
    if (s_print.session)
    {
        PMRelease(&s_print.session);
        s_print.session = 0;
    }
    
	return doPrint;
}

void PictProc(PicHandle pict)
{
    DrawPicture(pict, &(**pict).picFrame);
}

OSErr PageSetUp(int mode, PMPrintSession session, PMPageFormat *format)
{
short PrResFile;
GWorldPtr currPort;
GDHandle currDev;
Boolean pageSetupOK;
Handle pageFormatRes;
OSErr error;

	s_GetGWorld((GrafPtr*)&currPort, &currDev);
	s_SetGWorld(GetWindowPort(s_displayWindow), s_display);

	if (mode != s_samePageSetUp)
	{
		UseResFile(s_MyResFile);
		pageFormatRes = s_print.flattenPageFormat = GetResource('pgFm', 128);
		UseResFile(PrResFile);
		if (s_print.flattenPageFormat)
        {
            error = PMUnflattenPageFormat(s_print.flattenPageFormat, format);
            if (!error)
                error = PMSessionValidatePageFormat(session, *format, 
                            kPMDontWantBoolean);
            if (error)
                s_print.flattenPageFormat = 0;
        }
        
		if (!s_print.flattenPageFormat)
        {
			error = PMCreatePageFormat(format);
            if (!error)
                error = PMSessionDefaultPageFormat(session, *format);
		}
  	}
	if (!error && mode==s_noBlind)
        error = PMSessionPageSetupDialog(session, *format, &pageSetupOK);
    if (!error && pageSetupOK)
    {
        error = PMFlattenPageFormat(*format, &s_print.flattenPageFormat);
        UseResFile(s_MyResFile);
        if (!pageFormatRes)
            AddResource((Handle)s_print.flattenPageFormat,'pgFm', 128,"\pSCL OS X Print Record");
        else
            ChangedResource((Handle)s_print.flattenPageFormat);
        WriteResource((Handle)s_print.flattenPageFormat);
        UseResFile(PrResFile);
    }
	HPurge((Handle)s_print.flattenPageFormat);
	s_SetGWorld((GrafPtr)currPort, currDev);
	
	return error;
}

void DoPrintJob(void (*drawProc)(PicHandle), PicHandle pict, int textFlag)
{
PMRect pmRect;
OSErr error;
PMPrintSession session;
PMPrintSettings setting;
PMPageFormat format;

	if (textFlag)
	{
		session = s_print.textSession;
		setting = s_print.textSetting;
		format = s_print.textFormat;
	}
	else
	{
		session = s_print.session;
		setting = s_print.setting;
		format = s_print.format;
	}

    error = PMGetAdjustedPageRect(format, &pmRect);
    if (!error)
        error = PMSessionBeginDocument(session,
                            setting, format);
    if (!error && !PMSessionBeginPage(session,
                        format, &pmRect))
    {
        (*drawProc)(pict);
        PMSessionEndPage(session);
    }
    PMSessionEndDocument(session);
}


#else /*S_CARBON*/
int	PrintPict(PicHandle pict,int mode)
{
TPrStatus  stRec;
int doPrint;
short PrResFile;
GWorldPtr currPort;
GDHandle currDev;
TPPrPort prPort;

	s_GetGWorld((GrafPtr*)&currPort, &currDev);
	s_SetGWorld(s_displayWindow, s_display);
	PrOpen();
	PrResFile = CurResFile();
	if (mode != s_samePageSetUp)
	{
		UseResFile(s_MyResFile);
		s_print.prRecHdl = (THPrint)GetResource('PREC', 128);
		UseResFile(PrResFile);
		if ((!s_print.prRecHdl)||PrValidate(s_print.prRecHdl))
		{
			s_print.prRecHdl = (THPrint)NewHandle((long)sizeof(TPrint));
			PrintDefault(s_print.prRecHdl);
			UseResFile(s_MyResFile);
			AddResource((Handle)s_print.prRecHdl,'PREC', 128,"\pSCL Print Record");
			UseResFile(PrResFile);
		}
	}
	HNoPurge((Handle)s_print.prRecHdl);
	if (mode==s_noBlind) PrStlDialog(s_print.prRecHdl);
	if (mode==s_noBlind)
		doPrint = PrJobDialog(s_print.prRecHdl);
	else
		doPrint = 1;
	if(doPrint)
	{
		prPort = PrOpenDoc(s_print.prRecHdl, nil, nil);
		SetPort(&prPort->gPort);
		PrOpenPage(prPort, nil);
		DrawPicture(pict,&(**pict).picFrame);
		PrClosePage(prPort);
		PrCloseDoc(prPort);
		PrPicFile(s_print.prRecHdl, nil, nil, nil, &stRec);
		UseResFile(s_MyResFile);
		ChangedResource((Handle)s_print.prRecHdl);
		WriteResource((Handle)s_print.prRecHdl);
		UseResFile(PrResFile);
	}
	PrClose();
	HPurge((Handle)s_print.prRecHdl);
	s_SetGWorld((GrafPtr)currPort,currDev);
	
	return doPrint;
}

OSErr	PageSetUp(int mode)
{
GrafPtr	savePort;
int doPrint;
short PrResFile;

	GetPort(&savePort);
	PrOpen();
	PrResFile = CurResFile();
	UseResFile(s_MyResFile);
	s_print.prRecHdl = (THPrint)GetResource('PREC',128);
	UseResFile(PrResFile);
	if ((!s_print.prRecHdl)||PrValidate(s_print.prRecHdl))
	{
		s_print.prRecHdl = (THPrint)NewHandle((long)sizeof(TPrint));
		PrintDefault(s_print.prRecHdl);
		UseResFile(s_MyResFile);
		AddResource((Handle)s_print.prRecHdl,'PREC',128,"\pSCL Print record");
		UseResFile(PrResFile);
	}
	PrStlDialog(s_print.prRecHdl);
	PrClose();
	SetPort(savePort);
	CheckPause();
}
#endif /*S_CARBON*/

#endif /*!S_UNIX*/ /******************** UNIX ********************/

#if S_CARBON
void	BeginBitMapPrinting(void)
{
GWorldPtr currPort;
GDHandle currDev;
OSErr error;

    if (PMCreateSession(&s_print.session))
    {
        s_print.bitMapDoPrint = 0;
        return;
    }
    
	s_print.prResFile = CurResFile();

    error = PageSetUp(s_noBlind, s_print.session, &s_print.format);

	s_GetGWorld((GrafPtr*)&currPort, &currDev);
	s_SetGWorld(GetWindowPort(s_displayWindow), s_display);

    error = PMCreatePrintSettings(&s_print.setting);
    if (!error)
        error = PMSessionDefaultPrintSettings(s_print.session, s_print.setting);
    if (!error)
        error = PMSessionPrintDialog(s_print.session, s_print.setting, s_print.format, &s_print.bitMapDoPrint);

	s_SetGWorld(currPort, currDev);
    if (error)
    {
        s_print.bitMapDoPrint = 0;
        return;
    }
}

#elif !S_UNIX /*S_CARBON*/

void	BeginBitMapPrinting(void)
{
    PageSetUp(s_noBlind);
	PrOpen();
    s_print.bitMapDoPrint = PrJobDialog(s_print.prRecHdl);
	PrClose();
	CheckPause();
}
#else /*elif !S_UNIX  S_CARBON*/
void	BeginBitMapPrinting(void)
{
}
#endif /*else !S_CARBON*/

#if !S_UNIX /******************** !!!! UNIX ********************/
void	BeginBitMapPrintingSamePageSetUp(void)
{
	s_print.bitMapDoPrint = 1;
	CheckPause();
}

#endif /*!S_UNIX*/ /******************** UNIX ********************/
#if S_CARBON
void	EndBitMapPrinting(void)
{
	if(s_print.bitMapDoPrint)
        DoPrintJob(&BitMapProc, 0, 0);
    if (s_print.setting)
    {
        PMRelease(&s_print.setting);
        s_print.setting = 0;
    }
    if (s_print.session)
    {
        PMRelease(&s_print.session);
        s_print.session = 0;
    }
}

void BitMapProc(PicHandle pict)
{
GrafPtr pmPort;

    PMSessionGetGraphicsContext(s_print.session, kPMGraphicsContextQuickdraw,
            (void **)&pmPort);

    ForeColor(blackColor);
    BackColor(whiteColor);
	CopyBits(GetWindowBits(s_drawingWindow), GetPortBitMapForCopyBits(pmPort),
		SCWindowBounds(s_drawingWindow), SCWindowBounds(s_drawingWindow), srcCopy, 0);
}

void	BeginTextPrinting(void)
{
OSErr	error;
//GrafPtr	prPort;
GWorldPtr currPort;
GDHandle currDev;
//GrafPtr port;
PMRect pmRect;

	s_GetGWorld((GrafPtr*)&currPort, &currDev);
	s_SetGWorld(GetWindowPort(s_displayWindow), s_display);
    if (PMCreateSession(&s_print.textSession))
        return;
	s_print.prResFile = CurResFile();
    error = PageSetUp(s_noBlind, s_print.textSession, &s_print.textFormat);

    if (!(error = PMCreatePrintSettings(&s_print.textSetting)))
        error = PMSessionDefaultPrintSettings(s_print.textSession, s_print.textSetting);
	if (!error)
		error = PMSessionPrintDialog(s_print.textSession, s_print.textSetting,
                    s_print.textFormat, &s_textPrintFlag);
	//s_SetGWorld((GrafPtr)currPort, currDev);
	if (s_textPrintFlag)
	{
		PrintTextP = &PrintText;
		error = PMGetAdjustedPageRect(s_print.textFormat, &pmRect);
		s_print.textRect.top = pmRect.top;
		s_print.textRect.left = pmRect.left;
		s_print.textRect.bottom = pmRect.bottom;
		s_print.textRect.right = pmRect.right;
        //error = PMSessionBeginDocument(s_print.textSession,
                            //s_print.textSetting, s_print.textFormat);
		if (!error)
		{
			//error = PMSessionGetGraphicsContext(s_print.textSession,
						//kPMGraphicsContextQuickdraw, (void **)&prPort);
			//GetPort(&prPort);
			//if (!error)
			//{
				SetTextPrintTE();
			//}
			//PMSessionEndDocument(s_print.textSession);
		}
		if(!s_print.printWindow)
			s_print.printWindow=NewWindow(nil,&s_inputBounds, (unsigned char *)"\pPrint",
										false, altDBoxProc, (WindowPtr)-1L, true, 0);
		(**s_print.printH).inPort = GetWindowPort(s_print.printWindow);
	}
	
	s_SetGWorld((GrafPtr)currPort, currDev);
}

#else /*S_CARBON*/

void	EndBitMapPrinting(void)
{
#if !S_UNIX
TPrStatus  stRec;
GrafPtr	savePort;
short PrResFile;
TPPrPort prPort;

	if(s_print.bitMapDoPrint)
	{
		GetPort(&savePort);
		PrOpen();
		PrResFile = CurResFile();
		UseResFile(s_MyResFile);
		LoadResource((Handle)s_print.prRecHdl);
		UseResFile(PrResFile);
		HNoPurge((Handle)s_print.prRecHdl);
		prPort = PrOpenDoc(s_print.prRecHdl, nil, nil);
		SetPort(&prPort->gPort);
		PrOpenPage(prPort, nil);
		CopyBits(&s_drawingWindow->portBits, &(prPort->gPort).portBits,
			 &s_drawingWindow->portRect, &s_drawingWindow->portRect, srcCopy, nil);
		PrClosePage(prPort);
		PrCloseDoc(prPort);
		PrPicFile(s_print.prRecHdl, nil, nil, nil, &stRec);
		UseResFile(s_MyResFile);
		ChangedResource((Handle)s_print.prRecHdl);
		WriteResource((Handle)s_print.prRecHdl);
		UseResFile(PrResFile);
		HPurge((Handle)s_print.prRecHdl);
		PrClose();
		SetPort(savePort);
	}
	CheckPause();
#else/*!S_UNIX*/
char command[50];

	if (s_UNIX.drawingFlag)
	{
		fprintf(s_UNIX.drawingFile, "EndBitmapPrinting();\n");
		if (s_Config.XFlag)
		{
			sprintf(command, "xwd -id %d|xpr|lp", thePort->win);
			system(command);
		}
	}
#endif /*else !S_UNIX*/
}


void	BeginTextPrinting(void)
{
#if !S_UNIX
short	s_fontNum;
short PrResFile;
OSErr err;
TPPrPort prPort;
GrafPtr currPort;
GDHandle currDev;
#else
char userReply[256];
#endif

#if !S_UNIX
	s_GetGWorld((GrafPtr*)&currPort, &currDev);
	s_SetGWorld(GetWindowPort(s_displayWindow), s_display);
	PrOpen();
	PrResFile = CurResFile();
	UseResFile(s_MyResFile);
	s_print.textRecHdl = (THPrint)GetResource('PREC',130);
	UseResFile(PrResFile);
	if ((!s_print.textRecHdl)||PrValidate(s_print.textRecHdl))
	{
		s_print.textRecHdl = (THPrint)NewHandle((long)sizeof(TPrint));
		
		if (!s_print.textRecHdl)
		{
			s_OOMError();
		}
		
		PrintDefault(s_print.textRecHdl);
		UseResFile(s_MyResFile);
		AddResource((Handle)s_print.textRecHdl,'PREC',130,"\pSCL Print Text Record");
		UseResFile(PrResFile);
	}
	HNoPurge((Handle)s_print.textRecHdl);
	PrStlDialog(s_print.textRecHdl);
	s_textPrintFlag = PrJobDialog(s_print.textRecHdl);
	if(s_textPrintFlag)
	{
		PrintTextP = &PrintText;
		prPort = PrOpenDoc(s_print.textRecHdl, nil, nil);
		if (err = PrError())
		{
			if (err == iMemFullErr)
				s_OOMError();
			s_printf("Error in BeginTextPrinting()\nCode:%d\n", (int)err);
			Stop();
			exit(0);
		}
		s_print.textRect = prPort->gPort.portRect;
        SetTextPrintTE();

		PrCloseDoc(prPort);
	}
	PrClose();
	s_SetGWorld(currPort, currDev);
	if(!s_print.printWindow) s_print.printWindow=NewWindow(nil,&s_inputBounds,
									(unsigned char *)"\pPrint",false,altDBoxProc,
									(WindowPtr)-1L,true,0);
	(**s_print.printH).inPort = s_print.printWindow;
	CheckPause();

#else /*!S_UNIX*/

	if (s_Config.foreground)
	{
		printf("Do you start printing text? (y/n): ");
		fgets(userReply, 256, stdin);

		if ( CancelName(userReply) )
			return;
	}

	if ( s_print.textPrintFile = fopen( DataDirNameString ".SCLTextPrintSpool", "w") )
	{
		s_textPrintFlag=1;
		PrintTextP = &PrintText;
	}

#endif /*else !S_UNIX*/
}
#endif /*else S_CARBON*/

#if !S_UNIX

void SetTextPrintTE(void)
{
//GrafPtr savePort;

    //GetPort(&savePort);
    //SetPort(prPort);
    //GetPortBounds(prPort, &s_print.textRect);
    InsetRect(&s_print.textRect, 4, 0);
    TextFont(s_print.fontNum);
    TextSize(s_print.fontSize);
    s_print.printH = TENew(&s_print.textRect, &s_print.textRect);
    TEAutoView(FALSE, s_print.printH);
    s_print.maxLines = s_print.textRect.bottom/(**s_print.printH).lineHeight;
    (**s_print.printH).viewRect.bottom = s_print.maxLines*(**s_print.printH).lineHeight;
    (**s_print.printH).destRect.right -= 8;
    //SetPort(savePort);
}

void SetTextPrintFontName(char *name)
{
Str255 pName;

    C2PString(name, pName);
    GetFNum(pName, &s_print.fontNum);
}
void SetTextPrintFontSize(short size)
{
    s_print.fontSize = size;
}

void	EndTextPrinting(void)
{
//GWorldPtr currPort;
//GDHandle currDev;

	if (s_textPrintFlag)
	{
		//s_GetGWorld((GrafPtr*)&currPort, &currDev);
		//s_SetGWorld(GetWindowPort(s_displayWindow), s_display);

#if S_CARBON
		DoPrintJob(&TextPrintProc, 0, 1);
#else
		DoPrintJob(&TextPrintProc, 0, 1, s_print.textRecHdl);
#endif
		(**s_print.printH).inPort = GetWindowPort(s_print.printWindow);
		//SetPort(GetWindowPort(s_drawingWindow));
		s_textPrintFlag=0;
		PrintTextP=&s_voidDummyString;
		//s_SetGWorld((GrafPtr)currPort, currDev);
	}
#if !S_CARBON
	CheckPause();
#endif
}


void TextPrintProc(PicHandle pict)
{
GrafPtr port;
Rect r;

	GetPort(&port);
	(**s_print.printH).inPort = port;
	TextFont(s_print.fontNum);
	TextSize(s_print.fontSize);
	TEUpdate((GetPortBounds(port, &r)), s_print.printH);
}

#if !S_CARBON
void DoPrintJob(void (*drawProc)(PicHandle), PicHandle pict, int textFlag, THPrint recHandle)
{
TPrStatus  stRec;
GrafPtr	savePort;
short PrResFile;
TPPrPort prPort;
GWorldPtr currPort;
GDHandle currDev;

	s_GetGWorld((GrafPtr*)&currPort, &currDev);
	s_SetGWorld(GetWindowPort(s_displayWindow), s_display);
	PrOpen();
	PrResFile = CurResFile();
	UseResFile(s_MyResFile);
	LoadResource((Handle)recHandle);
	HNoPurge((Handle)recHandle);
	UseResFile(PrResFile);
	prPort = PrOpenDoc(recHandle, nil, nil);
	SetPort(&prPort->gPort);
#if 1
	PrOpenPage(prPort, nil);
	drawProc(pict);
	PrClosePage(prPort);
#endif
	PrCloseDoc(prPort);
	PrPicFile(s_print.textRecHdl, nil, nil, nil, &stRec);
	HPurge((Handle)recHandle);
	PrClose();
	s_SetGWorld((GrafPtr)currPort, currDev);
}
#endif /*!S_CARBON*/

#else /*!S_UNIX*/

void	EndTextPrinting(void)
{
	if (s_textPrintFlag)
	{
		fclose(s_print.textPrintFile);
		system("lp .SCLTextPrintSpool");
		/*remove(".SCLTextPrintSpool");*/ /*removed file is not printed*/
		s_textPrintFlag=0;
		PrintTextP=&s_voidDummyString;
	}
}
#endif /*else !S_UNIX*/

#if !S_UNIX
void PrintText(char* s)
{
//GWorldPtr currPort;
//GDHandle currDev;

	TEInsert(s,strlen(s), s_print.printH);
	if ( (**s_print.printH).nLines > s_print.maxLines)
	{
		//s_GetGWorld((GrafPtr*)&currPort, &currDev);
		//s_SetGWorld(GetWindowPort(s_displayWindow), s_display);

#if S_CARBON
		DoPrintJob(&TextPrintProc, 0, 1);
#else
		DoPrintJob(&TextPrintProc, 0, 1, s_print.textRecHdl);
#endif

		//(**s_print.printH).inPort = GetWindowPort(s_print.printWindow);
		//s_SetGWorld((GrafPtr)currPort, currDev);
		(**s_print.printH).inPort = GetWindowPort(s_print.printWindow);
		TESetText(*(TEGetText(s_print.printH))+(**s_print.printH).lineStarts[s_print.maxLines],
				(**s_print.printH).teLength-(**s_print.printH).lineStarts[s_print.maxLines],s_print.printH);
	}
}


#else /*!S_UNIX*/

void PrintText(char* s)
{
	fputs(s, s_print.textPrintFile);
}
#endif /*else UNIX*/

#if !S_UNIX /******************** !!!! UNIX ********************/
#include <s_define.c>
#endif /*!S_UNIX*/ /******************** UNIX ********************/


#endif /*pragma once*/
