/*
 *  s_Toolbox.h
 *  Version 2.0 / 010824
 *  by Satoshi Takahashi
 */

#ifndef S_TTOLBOX_H /*pragma once*/
#define S_TTOLBOX_H
#include <s_Config.h>
#if S_UNIX

#include <X11/Xlib.h>
/*dialog*/
#include <X11/Intrinsic.h> /* Boolean is defined here */
#include <s_ToolboxTypes.h>
#include <SCLXResource.h>

#define TBF 1 /* use two byte font */


/******************** QuickDraw ********************/

/********** Internal **********/

void WriteRect(Rect* rp);
void STSort( short *px, short *py );

/********** Line and Point**********/

void MoveTo(short x, short y);
void LineTo(short x, short y);
void Move(short x, short y);
void Line(short x, short y);
void GetPen(Point* pt);

/********** Rect **********/

void SetRect(Rect* rp, short left, short top, short right, short bottom);

void FrameRect(Rect* rp);
void PaintRect(Rect* rp);
void EraseRect(Rect* rp);
void InvertRect(Rect* rp);
void FillRect(Rect* rp, Pattern pat);


/********** Oval **********/

void FrameOval(Rect* rp);
void PaintOval(Rect* rp);
void EraseOval(Rect* rp);
void InvertOval(Rect* rp);
void FillOval(Rect* rp, Pattern pat);


/********** Text **********/

short TextWidth(char *string, short start, short length);
void DrawText(char* s1,short start,short length);
#if TBF
int DrawTextTwoByte(char* s1,short start,short length);
#endif
void DrawChar(char c);
void DrawString(Str255 string);


/********** Color **********/

#define blackColor		33
#define whiteColor		30
#define redColor		205
#define greenColor		341
#define blueColor		409
#define cyanColor		273
#define magentaColor	137
#define yellowColor		69

enum XColorName {blackXColor = 1, whiteXColor, 
	redXColor, greenXColor, blueXColor, cyanXColor,
	magentaXColor, yellowXColor};

void ForeColor(long color);
void BackColor(long color);
void RGBForeColor(RGBColor *colorP);
void RGBBackColor(RGBColor *colorP);


/********** Polygon **********/

PolyHandle OpenPoly(void);
void ClosePoly(void);
void KillPoly(PolyHandle poly);
void FramePoly(PolyHandle poly);
void PaintPoly(PolyHandle poly);
void ErasePoly(PolyHandle poly);
void FillPoly(PolyHandle poly, Pattern pat);


/********** Region **********/


RgnHandle NewRgn(void);
void OpenRgn(void);
void CloseRgn(RgnHandle rgnNo);
void PaintRgn(RgnHandle rgnNo);
void SectRgn(RgnHandle rgn1, RgnHandle rgn2, RgnHandle rgn3);


/********** Pen **********/

void PenSize(short h, short v);
void PenPat(Pattern pat);
void GetPenState(PenState *psP);
void SetPenState(PenState *psP);
void PenMode(int mode);
void HidePen(void);
void ShowPen(void);


/********** Other QuickDraw **********/

void ScrollRect(Rect *scrollRectP, short dh, short dv, RgnHandle updateRgn);
void GetIndPattern(Pattern pat, int patListID, int index);
void GlobalToLocal(Point *pt);
void GetCPixel(short x, short y, RGBColor *pixelP);
RGBColor *GetPortForeColor(CGrafPtr port, RGBColor *cp);
RGBColor *GetPortBackColor(CGrafPtr port, RGBColor *cp);

/********** Font **********/
void GetFNum(char *fontName, short *fontNumP);
OSStatus FMGetFontFamilyName(short font, Str255 fontNameP);
int ChangeXFont(void);
void TBSetStyleWord(void);
void TextFont(short fontNum);
void TextSize(short fontSize);
void TextFace(Style face);


/******************** Events ********************/

int GetNextEvent(long eventMask, EventRecord *eventP);
void GetMouse(Point *pen);


/******************** NewWindow ********************/

WindowPtr NewWindow(Ptr wStorage, Rect *boundsRect,
                        char *title, Boolean visible,
                        int theProc, WindowPtr behind,
                        Boolean goAwayFlag, long refcon);
WindowPtr NewCWindow(Ptr wStorage, Rect *boundsRect,
                        char *title, Boolean visible,
                        int theProc, WindowPtr behind,
                        Boolean goAwayFlag, long refcon);


/******************** Other Toolbox********************/

long TickCount(void);

#define NewPtr malloc

#define ExitToShell() exit(EXIT_SUCCESS)

#define DisposeHandle(a) (void)(0) /* dummy */



typedef int DateForm;

void GetDateTime(unsigned long *secP);
void IUDateString(long sec, DateForm format,
			Str255 dateString);
void IUTimeString(long sec, Boolean secFlag,
			Str255 timeString);
void DateString(long sec, DateForm format,
			Str255 dateString, Handle h);
void TimeString(long sec, Boolean secFlag,
			Str255 timeString, Handle h);
void SystemTask(void);

/******************************************
	Parameters Dialog
	uses X Toolkit
*******************************************/

DialogPtr GetNewDialog(short resID, Ptr dStorage, WindowPtr behind);
void SXGetDialogResourceInfo(SXDialogResourceItem dialogRes,
								DialogPeek newDialog);
int SetDialogItems(DialogPeek newDialog, short resID);
void SXDItemToWidget(int resID, int itemN, DialogPeek dialog, DITLItem *dItem,
						Cardinal num_args, Arg args[]);
void SXDItemHit(Widget widget, XtPointer clientData, XtPointer callerData);
void SXEditTextCallback(Widget widget, XtPointer clientData,
								 XtPointer callerData);
void SXEditButtonCallback(Widget widget, XtPointer clientData,
								 XtPointer callerData);
SXEditButtonPopupPtr SXMakeEditButtonPopup(void);
void SXPopdownText(Widget widget, XtPointer clientData,
								 XtPointer callerData);
void SXAccelerate(Widget w, Widget shell);

ControlHandle NewControl(WindowPtr theWindow, Rect *ctrlRect,
		Str255 title, Boolean visible, short initialValue,
		short minValue, short maxValue, short ctrlType,
		long refCon);

void GetDialogItem(DialogPtr theDialog, short theItem, short *iType,
		Handle *iHandle, Rect *iRect);
void SetDialogItem(DialogPtr theDialog, short theItem, short iType,
		Handle iHandle, Rect *iRect);
void GetDialogItemText(Handle iHandle, Str255 textStr);
void SetDialogItemText(Handle iHandle, Str255 textStr);
short GetControlValue(ControlHandle control);
void SetControlValue(ControlHandle control, short value);

void SelectDialogItemText(DialogPtr dialog, short item, short from, short to);

void ModalDialog(ProcPtr filter, short *itemHit);
void SelectWindow(WindowPtr window);
void ShowWindow(WindowPtr window);
void HideWindow(WindowPtr window);
void SXPopUpDialog(WindowPtr dialog);
void SXPopDownDialog(WindowPtr dialog);

/***** Memory *****/
void HLock(void *handle);
void HUnlock(void *handle);


#endif /*S_UNIX*/
#endif /*pragma once*/
